package fr.janua.client;

//import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.web.bindery.event.shared.SimpleEventBus;
import com.google.web.bindery.requestfactory.shared.Receiver;
import com.google.web.bindery.requestfactory.shared.Request;
import com.mvp4g.client.annotation.EventHandler;
import com.mvp4g.client.event.BaseEventHandler;

import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.proxy.UserProxy;
import fr.janua.client.stub.UserRequest;
import fr.janua.shared.Action;
import fr.janua.shared.Log;

/**
 * Singleton permettant de gérer les appels vers le serveur, et l'état de la connexion.
 * Exemple d'utilisation :
 * <code>AppControllManager.getInstance().getUsername();</code>
 */
@EventHandler
public class AppControllManager extends BaseEventHandler<JaguardsEventBus> {
	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	private CrudRequestFactory requestFactory = GWT.create(CrudRequestFactory.class);
	private LogServiceAsync logService = GWT.create(LogService.class);
	private CrudServiceAsync crudService = GWT.create(CrudService.class);

	private static final int TIMEOUT = 15; // 15 min
	private Timer timer;

	/**
	 * Lance le timer, ou le prolonge.
	 */
	public void onActivity(){
		/*
		 *           +-------------+---------------+
		 *           |  connecté   |   déconnecté  |
		 *	---------+-------------+---------------+
		 *   timer   |  instancier |    ne rien    |
		 *    null   |    timer    |     faire     |
		 *  ---------+-------------+---------------+
		 *   timer   |  prolonger  |    stoper     |
		 *   ! null  |    timer    |     timer     |
		 *  ---------+-------------+---------------+
		 */
		if(UserManager.getInstance().isLogged()){
			if(timer == null){
				// instanciation du Timer
				timer = new Timer() {
					@Override
					public void run() {
						eventBus.addLog(Action.DISCONNECTION, "timeout [" + TIMEOUT + "min]");
						UserManager.getInstance().setCurrentUser(null);
						eventBus.goToPageLogin();
						eventBus.setStatusConnection(null, false);
						Window.alert(constants.logoutInactivity());
					}
				};
				timer.schedule(TIMEOUT * 60 * 1000);
			}else{
				// prolongation du Timer
				timer.schedule(TIMEOUT * 60 * 1000);
			}
		}else if (timer != null){
			// stoper timer
			timer.cancel();
		}
	}

	/**
	 * Ajoute un message de LOG dans la base de données. Cette méthode utilise le système RPC
	 * @param action
	 * @param detail
	 */
	public void onAddLog(Action action, String detail){
		if (logService == null) {
			logService = GWT.create(LogService.class);
		}

		Log newLog = new Log();
		newLog.setLoginAuthor(UserManager.getInstance().getLogin() + " (" + UserManager.getInstance().getGroupName() + ")");
		newLog.setAction(action.name());
		newLog.setDetail(detail);


		// Set up the callback object.
		AsyncCallback<Void> callback = new AsyncCallback<Void>() {
			public void onFailure(Throwable caught) {
				Window.alert("log error : " + caught.toString());
			}

			public void onSuccess(Void v) {
			}
		};

		// Make the call to the stock price service.
		logService.addLog(newLog, callback);
	}

	public void onRefreshLastActivityUser(){
		final String login = UserManager.getInstance().getLogin();
		final String password = UserManager.getInstance().getPassword();

		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		UserRequest userRequest = requestFactory.getUserRequest();
		Request<UserProxy> request = userRequest.checkWithEncryptedPaswword(login, password).with("group", "siteAllowed", "group.siteAllowed", "group.typeAllowed");
		request.fire(new Receiver<UserProxy>() {
			@Override
			public void onSuccess(UserProxy response) {
				UserManager.getInstance().setCurrentUser(response);
				if(response == null){
					// connection fail
					Window.alert(constants.failRefreshUserPleaseReconnect());
					// statut affiché en haut à droite
					eventBus.setStatusConnection("", false);
					eventBus.goToPageLogin();
					eventBus.addLog(Action.CONNECTION, "refresh user fail [" + login + "]");
				}else{
					// Le currentUser a bien était actualisé. La page MainCourante va alors se recharger.
					eventBus.goToPageMainCourante();
				}
			}
		});

	}
	
	public void onRefreshUserWithoutChangePassword() {
		final String login = UserManager.getInstance().getLogin();
		final String password = UserManager.getInstance().getPassword();

		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		UserRequest userRequest = requestFactory.getUserRequest();
		Request<UserProxy> request = userRequest.checkWithEncryptedPaswword(login, password).with("group", "siteAllowed", "group.siteAllowed", "group.typeAllowed");
		request.fire(new Receiver<UserProxy>() {
			@Override
			public void onSuccess(UserProxy response) {
				UserManager.getInstance().setCurrentUser(response);
				if(response == null){
//					Window.alert("refresh fail avec (" + login + ", " + password + ")");
					// connection fail
					Window.alert(constants.failRefreshUserPleaseReconnect());
					// statut affiché en haut à droite
					eventBus.setStatusConnection("", false);
					eventBus.goToPageLogin();
					eventBus.addLog(Action.CONNECTION, "refresh user fail [" + login + "]");
				}else{
//					Window.alert("refresh success !");
					// refresh success
				}
			}
		});

	}

	/**
	 * A la modification de l'utilisateur, de son groupe, d'un site ou d'un type de son groupe, on refresh l'utilisateur courant.
	 * Il est possible de se faire déconnecter si quelqu'un a changé notre mot de passe !
	 * @param login
	 * @param password
	 */
	public void onRefreshUser(final String login, final String password) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		UserRequest userRequest = requestFactory.getUserRequest();
		Request<UserProxy> request = userRequest.check(login, password).with("group", "siteAllowed", "group.siteAllowed", "group.typeAllowed");
		request.fire(new Receiver<UserProxy>() {
			@Override
			public void onSuccess(UserProxy response) {
				UserManager.getInstance().setCurrentUser(response);
				if(response == null){
//					Window.alert("refresh fail avec (" + login + ", " + password + ")");
					// connection fail
					Window.alert(constants.failRefreshUserPleaseReconnect());
					// statut affiché en haut à droite
					eventBus.setStatusConnection("", false);
					eventBus.goToPageLogin();
					eventBus.addLog(Action.CONNECTION, "refresh user fail [" + login + "]");
				}else{
//					Window.alert("refresh success !");
					// refresh success
				}
			}
		});
	}


	/**
	 * Recharge le fichier de configuration de la base de données.
	 */
	public void onReloadDB(){
		if (crudService == null) {
			crudService = GWT.create(CrudService.class);
		}

		
		// Set up the callback object.
		AsyncCallback<Void> callback = new AsyncCallback<Void>() {
			public void onFailure(Throwable caught) {
				Window.alert("reload DB error : " + caught.toString());
			}

			public void onSuccess(Void v) {
				// recharge la Page Login
				eventBus.goToPageLogin();
			}
		};

		// Make the call to the stock price service.
		crudService.reloadConfDB(callback);
	}



}