package fr.janua.client.presenter;

import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.mvp4g.client.annotation.Presenter;
import com.mvp4g.client.presenter.LazyPresenter;

import fr.janua.client.CrudService;
import fr.janua.client.JaguardsEventBus;
import fr.janua.client.LogService;
import fr.janua.client.LogServiceAsync;
import fr.janua.client.view.PageLogView;
import fr.janua.client.view.interfaces.IPageLogView;
import fr.janua.client.view.interfaces.IPageLogView.IPageLogPresenter;
import fr.janua.shared.Log;


@Presenter( view = PageLogView.class )
public class PageLogPresenter extends LazyPresenter<IPageLogView, JaguardsEventBus> implements IPageLogPresenter {
	private LogServiceAsync logService = GWT.create(LogService.class);
	private int nbElemPerPage = 20;
	private int currentPage;
	private int nbLog;

	public void onGoToPageLog(){
		eventBus.setBody( view );
		view.loadPage();
	}

	@Override
	public void generatePagination() {
		if (logService == null) {
			logService = GWT.create(CrudService.class);
		}

		AsyncCallback<Number> callback = new AsyncCallback<Number>() {
			public void onFailure(Throwable caught) {
				Window.alert("generatePagination error : " + caught.toString());
			}

			public void onSuccess(Number result) {
				int nbLogReceive = result.intValue();
				nbLog = nbLogReceive;
				int nbPage = getNbPage(nbLog, nbElemPerPage);
				currentPage = 1;
				view.displayPagination(currentPage, nbPage);
			}
		};

		logService.getNbLog(callback);
	}

	@Override
	public void getFirstPageLog() {
		getLog(0, nbElemPerPage);
	}

	/**
	 * Envoie une requête pour récupérer n Log. 
	 * @param startIndex
	 * @param nbElem
	 */
	private void getLog(int startIndex, int nbElem){
		if (logService == null) {
			logService = GWT.create(LogService.class);
		}

		AsyncCallback<List<Log>> callback = new AsyncCallback<List<Log>>() {
			public void onFailure(Throwable caught) {
				Window.alert("getLog error : " + caught.toString());
			}

			public void onSuccess(List<Log> result) {
				int nbLog = result.size();
				for(int i=0; i<nbLog; i++){
					view.displayOneLog(result.get(i));
				}
			}
		};

		logService.getLog(startIndex, nbElem, callback);
	}


	/**
	 * Retourne le nombre de page que l'on doit afficher en fonction du nombre de Log et du nombre d'élément par page voulu.
	 * @param nbLog
	 * @param nbLogPerPage
	 * @return
	 */
	private int getNbPage(int nbLog, int nbLogPerPage){
		int nbPage;
		if(nbLog%nbLogPerPage == 0)
			nbPage = (nbLog/nbLogPerPage);
		else
			nbPage = (nbLog/nbLogPerPage)+1;
		return nbPage;
	}

	@Override
	public void previousPage() {
		int nbPage = getNbPage(nbLog, nbElemPerPage);
		if(currentPage > 1)
			currentPage--;
		getLog((currentPage-1)*nbElemPerPage, nbElemPerPage);
		view.displayPagination(currentPage, nbPage);
	}

	@Override
	public void nextPage() {
		int nbPage = getNbPage(nbLog, nbElemPerPage);
		if(currentPage < nbPage)
			currentPage++;
		getLog((currentPage-1)*nbElemPerPage, nbElemPerPage);
		view.displayPagination(currentPage, nbPage);
	}

}