package fr.janua.client.presenter;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.web.bindery.event.shared.SimpleEventBus;
import com.google.web.bindery.requestfactory.shared.Receiver;
import com.google.web.bindery.requestfactory.shared.Request;
import com.google.web.bindery.requestfactory.shared.ServerFailure;
import com.mvp4g.client.annotation.Presenter;
import com.mvp4g.client.presenter.LazyPresenter;

import fr.janua.client.CrudRequestFactory;
import fr.janua.client.CrudService;
import fr.janua.client.CrudServiceAsync;
import fr.janua.client.JaguardsEventBus;
import fr.janua.client.UserManager;
import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.proxy.GroupProxy;
import fr.janua.client.proxy.SiteProxy;
import fr.janua.client.proxy.TypeProxy;
import fr.janua.client.proxy.UserProxy;
import fr.janua.client.stub.GroupRequest;
import fr.janua.client.stub.SiteRequest;
import fr.janua.client.stub.TypeRequest;
import fr.janua.client.stub.UserRequest;
import fr.janua.client.view.PageLoginView;
import fr.janua.client.view.interfaces.IPageLoginView;
import fr.janua.client.view.interfaces.IPageLoginView.IPageLoginPresenter;
import fr.janua.shared.Action;


@Presenter(view = PageLoginView.class)
public class PageLoginPresenter extends LazyPresenter<IPageLoginView, JaguardsEventBus> implements IPageLoginPresenter {

	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	private CrudRequestFactory requestFactory = GWT.create(CrudRequestFactory.class);
	private CrudServiceAsync crudService = GWT.create(CrudService.class);
	private boolean DEMO_MODE = true;

	public void onGoToPageLogin() {
		if(DEMO_MODE){
			resetGroupForDemoMode();
		}
		if (crudService == null) {
			crudService = GWT.create(CrudService.class);
		}

		AsyncCallback<Boolean> callback = new AsyncCallback<Boolean>() {
			public void onFailure(Throwable caught) {
				Window.alert("checkDB error : " + caught.toString());
			}

			public void onSuccess(Boolean result) {
				eventBus.setBody(view);
				view.loadPage();
				if(result){
					// BDD bien configurée
					view.setWarningVisible(false);
				}else{
					// BDD mal configuré
					view.setWarningVisible(true);
				}
			}
		};

		crudService.checkDB(callback);
	}

	@Override
	public void checkSignIn(final String login, String password) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		UserRequest userRequest = requestFactory.getUserRequest();
		Request<UserProxy> request = userRequest.check(login, password).with("group", "siteAllowed", "group.siteAllowed", "group.typeAllowed");
		request.fire(new Receiver<UserProxy>() {
			@Override
			public void onSuccess(UserProxy user) {
				if(user == null){
					// connection fail : mauvais mot de passe
					Window.alert(constants.badLoginPassword());
					// statut affiché en haut à droite
					eventBus.setStatusConnection("", false);
					eventBus.addLog(Action.CONNECTION, "fail [" + login + "]");
				}else if(!user.isEnabled()){
					// connection fail : compte désactivé
					Window.alert(constants.accountDisabled());
					// statut affiché en haut à droite
					eventBus.setStatusConnection("", false);
					eventBus.addLog(Action.CONNECTION, "fail [" + login + "]");
				}else{
					// connection success
					UserManager.getInstance().setCurrentUser(user);
					eventBus.activity();
					// efface les champs de saisis
					view.clearField();
					// statut affiché en haut à droite
					eventBus.setStatusConnection(user.getLogin(), true);
					// redirection page Main Courante
					eventBus.goToPageMainCourante();
					eventBus.addLog(Action.CONNECTION, "success");
				}
			}

			@Override
			public void onFailure(ServerFailure error) {
				// connection fail : login inconnu
				Window.alert(constants.badLoginPassword());
				// statut affiché en haut à droite
				eventBus.setStatusConnection("", false);
				eventBus.addLog(Action.CONNECTION, "fail [" + login + "]");
			}
		});
	}

	@Override
	public void initDbStep1() {
		if(requestFactory.getEventBus() == null)
			requestFactory.initialize(new SimpleEventBus());
		String[] tabNameSite = new String [] {"Nice", "Rome", "Paris", "Bruxelles", "Berlin", "Toulon"};
		String[] tabNameType = new String [] {"Incendie", "Vol", "Bagarre", "Inondation", "Expulsion", "Incivilité", "Police"};
		String[] tabBgColor = new String [] {"#FF4500", "#008080", "#000080", "#00BFFF", "#9370D8", "#FA8072", "#FFD700"};
		String[] tabTextColor = new String [] {"whiteSmoke", "black", "whiteSmoke", "whiteSmoke", "black", "black", "black"};

		createGroupAdmin();

		creationSite(tabNameSite);

		creationType(tabNameType, tabBgColor, tabTextColor);

	}

	@Override
	public void initDbStep2() {
		if(requestFactory.getEventBus() == null)
			requestFactory.initialize(new SimpleEventBus());

		createUserAdmin();

		SiteRequest siteRequest = requestFactory.getSiteRequest();
		Request<List<SiteProxy>> request = siteRequest.findAllSite();

		request.fire(new Receiver<List<SiteProxy>>() {
			@Override
			public void onSuccess(List<SiteProxy> response) {

				final Set<SiteProxy> s = new HashSet<SiteProxy>(response);

				final GroupRequest groupRequest1 = requestFactory.getGroupRequest();
				final Request<GroupProxy> request = groupRequest1.findByName("admin");

				request.fire(new Receiver<GroupProxy>() {
					@Override
					public void onSuccess(GroupProxy response) {
						// 2 - on enlève le frozen de response, et on crée un 2nd context appelé groupRequest2(groupRequest1 étant occupé)
						GroupRequest groupRequest2 = requestFactory.getGroupRequest();
						GroupProxy groupUpdated = groupRequest2.edit(response);

						// 3 - on modifie le GroupProxy récupéré dans la BDD
						groupUpdated.setSiteAllowed(s);

						// 4 - on le renvoie avec les nouvelle valeurs
						Request<Boolean> request2 = groupRequest2.update(groupUpdated);

						request2.fire(new Receiver<Boolean>() {
							@Override
							public void onSuccess(Boolean response) {
								view.incrProgressAssoc();
								if(response){
									//									Window.alert("Mise à jour des Sites du Groupe effectuée.");
								}else{
									Window.alert("Une erreure est survenue. La mise à jour a échouée.");
								}
							}
						});
					}
				});
			}
		});		
	}

	@Override
	public void reloadDB() {
		eventBus.reloadDB();
	}

	/**
	 * S'il n'existe pas, créer un Group nommé "admin" ayant tous les droits.
	 */
	private void createGroupAdmin(){

		GroupRequest groupRequest = requestFactory.getGroupRequest();
		Request<GroupProxy> request = groupRequest.findByName("Administrator");
		request.fire(new Receiver<GroupProxy>() {
			@Override
			public void onSuccess(GroupProxy response) {
				view.incrProgressAdmin();
				if(response==null){
					GroupRequest groupRequest = requestFactory.getGroupRequest();

					// Création du Group "admin"
					GroupProxy groupProxy = groupRequest.create(GroupProxy.class);
					groupProxy.setName("Administrator");
					groupProxy.setMail("phan.gilles@gmail.com");
					groupProxy.setCrudGroup(7);
					groupProxy.setCrudMessage(7);
					groupProxy.setCrudSite(7);
					groupProxy.setCrudType(7);
					groupProxy.setCrudUser(7);
					groupProxy.setCrudLog(7);
					groupProxy.setCrudUploadDoc(7);

					Request<Boolean> requestGroup = groupRequest.create(groupProxy);
					requestGroup.fire();
					//					Window.alert("Admin crée");
				}else{
					//					Window.alert("Admin déjà crée");
				}
			} 
		});		
	}

	/**
	 * S'il n'existe pas, créer un User nommé "admin", du groupe "Administrator".
	 */
	private void createUserAdmin(){
		// 1 - on recherche s'il existe un User avec le login "Admin"
		UserRequest userRequest = requestFactory.getUserRequest();
		Request<UserProxy> request = userRequest.findByLogin("Admin");
		request.fire(new Receiver<UserProxy>() {
			@Override
			public void onSuccess(UserProxy response) {
				if(response==null){
					// 2 - s'il n'existe pas, on cherche le group "Administrator"
					GroupRequest groupRequest = requestFactory.getGroupRequest();
					Request<GroupProxy> request = groupRequest.findByName("Administrator");
					request.fire(new Receiver<GroupProxy>() {
						@Override
						public void onSuccess(GroupProxy group) {
							// 3 - si le Group existe, alors on peut créer l'utilisateur "Admin", et lui associer le groupe "Administrator". Limpide, n'est-ce pas ??
							if(group != null){
								UserRequest userRequest2 = requestFactory.getUserRequest();

								// Création du Group "admin"
								UserProxy userProxy = userRequest2.create(UserProxy.class);
								userProxy.setLogin("Admin");
								userProxy.setPassword("*");
								userProxy.setFirstName("Gilles");
								userProxy.setLastName("Phan");
								userProxy.setMail("phan.gilles@gmail.com");
								userProxy.setGroup(group);
								userProxy.setEnabled(true);

								Request<Boolean> requestGroup = userRequest2.create(userProxy);
								requestGroup.fire();
								view.incrProgressAdmin();
							}
						}
					});
				}else if(response.getLogin().equals("Admin")){
					view.incrProgressAdmin();
				}
			}
		});	
	}

	/**
	 * Créer les Site passé en paramètre s'ils n'existent pas.
	 * @param tabName
	 */
	private void creationSite(String[] tabName){
		// Pour chaque Site
		for(final String siteName : tabName){
			// Check s'il existe ou pas
			SiteRequest siteRequest = requestFactory.getSiteRequest();
			Request<SiteProxy> request = siteRequest.findByName(siteName);

			request.fire(new Receiver<SiteProxy>() {
				@Override
				public void onSuccess(SiteProxy response) {
					view.incrProgressSite();
					if(response == null){
						// S'il n'existent pas, on le crée
						SiteRequest siteRequest = requestFactory.getSiteRequest();
						final SiteProxy siteProxy = siteRequest.create(SiteProxy.class);
						siteProxy.setName(siteName);
						siteProxy.setEnabled(true);

						Request<Boolean> requestSite = siteRequest.create(siteProxy);
						requestSite.fire();
						//						Window.alert("Création du Site " + siteName + "!");
					}else{
						// Si non, on ne fait rien
						//						Window.alert("Site " + siteName + " déjà crée");
					}
				}
			});
		}
	}

	/**
	 * Création des Type passé en paramètre s'ils n'existent pas.
	 * @param tabName
	 * @param tabBgColor
	 * @param tabTextColor
	 */
	private void creationType(final String[] tabName, final String[] tabBgColor, final String[] tabTextColor){
		// Si les tableaux ont des tailles différentes (Transitivité !!!)
		if(tabName.length != tabBgColor.length || tabName.length != tabTextColor.length)
			return;
		// Pour chaque Type
		for(int i=0; i<tabName.length; i++){
			// Check s'il existe ou pas
			TypeRequest typeRequest = requestFactory.getTypeRequest();
			Request<TypeProxy> request = typeRequest.findByName(tabName[i]);
			final int iFinal = i;
			request.fire(new Receiver<TypeProxy>() {
				@Override
				public void onSuccess(TypeProxy response) {
					view.incrProgressType();
					if(response == null){
						// S'il n'existent pas, on le crée
						TypeRequest typeRequest = requestFactory.getTypeRequest();
						final TypeProxy typeProxy = typeRequest.create(TypeProxy.class);
						typeProxy.setName(tabName[iFinal]);
						typeProxy.setFgColor(tabTextColor[iFinal]);
						typeProxy.setBgColor(tabBgColor[iFinal]);
						typeProxy.setEnabled(true);

						Request<Boolean> requestType = typeRequest.create(typeProxy);
						requestType.fire();
					}else{
						// Si non, on ne fait rien
						//						Window.alert("#" + iFinal + " Type " + tabName[iFinal] + " déjà crée");
					}
				}
			});
		}
	}

	
	/**
	 * Réinitialise TOUS les groupes pour qu'ils puissent créer, éditer et voir les messages, et qu'ils n'aient que le droit de voir pour tout le reste.
	 */
	private void resetGroupForDemoMode() {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		GroupRequest groupRequest = requestFactory.getGroupRequest();
		Request<Void> request = groupRequest.resetForDemoMode();
		request.fire();
	}




















}