package fr.janua.client.presenter;

import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.web.bindery.event.shared.SimpleEventBus;
import com.google.web.bindery.requestfactory.shared.Receiver;
import com.google.web.bindery.requestfactory.shared.Request;
import com.mvp4g.client.annotation.Presenter;
import com.mvp4g.client.presenter.LazyPresenter;

import fr.janua.client.CrudRequestFactory;
import fr.janua.client.JaguardsEventBus;
import fr.janua.client.UserManager;
import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.proxy.SiteProxy;
import fr.janua.client.stub.SiteRequest;
import fr.janua.client.view.PageSiteView;
import fr.janua.client.view.interfaces.IPageSiteView;
import fr.janua.client.view.interfaces.IPageSiteView.IPageSitePresenter;
import fr.janua.shared.Action;


@Presenter( view = PageSiteView.class )
public class PageSitePresenter extends LazyPresenter<IPageSiteView, JaguardsEventBus> implements IPageSitePresenter {
	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	private CrudRequestFactory requestFactory = GWT.create(CrudRequestFactory.class);

	public void onGoToPageSite(){
		eventBus.setBody( view );
		view.loadPage();
	}

	@Override
	public void getAllSite() {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		SiteRequest osRequest = requestFactory.getSiteRequest();
		Request<List<SiteProxy>> request = osRequest.findAllSite();
		
		request.fire(new Receiver<List<SiteProxy>>() {
			@Override
			public void onSuccess(List<SiteProxy> response) {
				eventBus.addLog(Action.READ_SITE, "[all]");
				for(SiteProxy site : response){
					view.displayOneSite(site.getName(), site.isEnabled());
				}
			}
		});
	}

	@Override
	public void createSite(final String name, boolean isEnabled) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		SiteRequest siteRequest = requestFactory.getSiteRequest();
		
		SiteProxy siteProxy = siteRequest.create(SiteProxy.class);
		siteProxy.setName(name);
		siteProxy.setEnabled(isEnabled);
		
		Request<Boolean> request = siteRequest.create(siteProxy);
		request.fire(new Receiver<Boolean>() {
			@Override
			public void onSuccess(Boolean response) {
				if(response){
					eventBus.addLog(Action.CREATE_SITE, "échec [name:" + name + "]");
					view.loadPage();
					Window.alert(constants.successCreateSite());
				}else{
					eventBus.addLog(Action.CREATE_SITE, "succès [name:" + name + "]");
					Window.alert(constants.failCreateSite());
				}
			}
		});
	}

	@Override
	public void updateSite(final String name, final boolean isEnabled) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		final SiteRequest siteRequest1 = requestFactory.getSiteRequest();

		// 1 - on récupère le SiteProxy
		final Request<SiteProxy> request = siteRequest1.findByName(name);

		request.fire(new Receiver<SiteProxy>() {
			@Override
			public void onSuccess(SiteProxy response) {
				if(response == null){
					eventBus.addLog(Action.UPDATE_SITE, "échec, le site " + name + " n'existe pas");
					Window.alert(constants.locationDoesntExist());

				}else{
					// 2 - on enlève le frozen de response, et on crée un 2nd context appelé siteRequest2(siteRequest1 étant occupé)
					SiteRequest siteRequest2 = requestFactory.getSiteRequest();
					SiteProxy siteUpdated = siteRequest2.edit(response);
	
					// 3 - on modifie le SiteProxy récupéré dans la BDD
					siteUpdated.setEnabled(isEnabled);
	
					// 4 - on le renvoie avec les nouvelle valeurs
					Request<Boolean> request2 = siteRequest2.update(siteUpdated);
	
					request2.fire(new Receiver<Boolean>() {
						@Override
						public void onSuccess(Boolean response) {
							if(response){
								for(SiteProxy site : UserManager.getInstance().getSitesAllowed()){
									if(site.getName().equals(name)){
										eventBus.refreshUserWithoutChangePassword();
										break;
									}
								}
								eventBus.addLog(Action.UPDATE_SITE, "succès [name:" + name + "]");
								Window.alert(constants.successUpdateSite());
								view.loadPage();
							}else{
								eventBus.addLog(Action.UPDATE_SITE, "échec [name:" + name + "]");
								Window.alert(constants.failUpdateSite());
							}
						}
					});
				}
			}
		});
	}



}