package fr.janua.client.view;

import java.util.Date;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;

import fr.janua.client.UserManager;
import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.view.interfaces.IPageLogView;
import fr.janua.client.view.interfaces.IPageLogView.IPageLogPresenter;
import fr.janua.shared.Action;
import fr.janua.shared.Log;

/**
 * 
 * @author Gilles Phan
 *
 */
public class PageLogView extends ReverseCompositeView<IPageLogPresenter> implements IPageLogView {
	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	
	private static PageLogViewUiBinder uiBinder = GWT.create(PageLogViewUiBinder.class);

	interface PageLogViewUiBinder extends UiBinder<Widget, PageLogView> {}

	public PageLogView() {
		initWidget(uiBinder.createAndBindUi(this));
	}

	
	@UiField Label label_error;
	
	@UiField FlowPanel fPanel_listLog;

	// pagination du haut et du bas
	@UiField Button button_previous1, button_next1, button_previous2, button_next2;
	@UiField Label label_numPage1, label_numPage2;

	/* Override */

	@Override
	public void createView() {
	}

	@Override
	public void loadPage() {
		// reset tous les champs
		fPanel_listLog.clear();
		
		if(UserManager.getInstance().isViewableLog()){
			// Si droits suffisant
			label_error.setVisible(false);
			// générer la pagination (récupère le nombre de log, et affiche en fonction du nombre par page)
			presenter.generatePagination();
			presenter.getFirstPageLog();
		}else{
			// si non, signaler qu'on n'a pas les droits nécessaires...
			// ... on masque l'interface de navigation, vu qu'il n'est pas fait par "presenter.generatePagination();" 
			label_numPage1.setVisible(false);
			label_numPage2.setVisible(false);
			button_previous1.setVisible(false);
			button_previous2.setVisible(false);
			button_next1.setVisible(false);
			button_next2.setVisible(false);
			label_error.setVisible(true);
		}
		
	}

	@Override
	public void displayOneLog(Log log){
		
		FlowPanel fPanelDate = new FlowPanel();
		FlowPanel fPanelAction = new FlowPanel();
		FlowPanel fPanelIp = new FlowPanel();
		FlowPanel fPanelDetails = new FlowPanel();
		
		fPanelDate.setStyleName("span3");
		fPanelAction.setStyleName("span2");
		fPanelIp.setStyleName("span2");
		fPanelDetails.setStyleName("span5");
		
		fPanelDate.add(new Label(longToTime(log.getDate())));
		fPanelAction.add(new Label( Action.valueOf(log.getAction()).toString() ));
		fPanelIp.add(new Label(log.getIp()));
		fPanelDetails.add(new Label(log.getLoginAuthor() + " : " + log.getDetail()));
		
		FlowPanel fPanelWrapperLog = new FlowPanel();
		fPanelWrapperLog.addStyleName("row-fluid highlight-background");
		
		fPanelWrapperLog.add(fPanelDate);
		fPanelWrapperLog.add(fPanelAction);
		fPanelWrapperLog.add(fPanelIp);
		fPanelWrapperLog.add(fPanelDetails);
		
		fPanel_listLog.add(fPanelWrapperLog);

	}

	@Override
	public void displayPagination(int currentPage, int nbPage) {
		if(nbPage<=1){
			label_numPage1.setVisible(false);
			label_numPage2.setVisible(false);
			button_previous1.setVisible(false);
			button_previous2.setVisible(false);
			button_next1.setVisible(false);
			button_next2.setVisible(false);
		}else{
			label_numPage1.setVisible(true);
			label_numPage2.setVisible(true);
			button_previous1.setVisible(true);
			button_previous2.setVisible(true);
			button_next1.setVisible(true);
			button_next2.setVisible(true);
			
			label_numPage1.setText(currentPage + "/" + nbPage);
			label_numPage2.setText(currentPage + "/" + nbPage);
			
			button_previous1.removeStyleName("disabled");
			button_previous2.removeStyleName("disabled");
			
			button_next1.removeStyleName("disabled");
			button_next2.removeStyleName("disabled");
			if(currentPage<=1){
				button_previous1.addStyleName("disabled");
				button_previous2.addStyleName("disabled");
			}
			if(currentPage>=nbPage){
				button_next1.addStyleName("disabled");
				button_next2.addStyleName("disabled");
			}
		}
	}

	@Override
	public void clearPanelLog(){
		fPanel_listLog.clear();
	}
	
	
	/* Fonction */

	/**
	 * Convertit une date de type long vers une String formaté pour donner la date correspondante.
	 * Format : "30 avril 2013 à 11:16:25"
	 * @param time
	 * @return
	 */
	private String longToTime(long time){
		String[] tabMonth = {"error Date", constants.january(), constants.febuary(), constants.march(), constants.april(), constants.may(), constants.june(), constants.july(), constants.august(), constants.september(), constants.october(), constants.november(), constants.december()};
		String res = "";
		Date date = new Date(time);

		// 30 ...
		DateTimeFormat fmt = DateTimeFormat.getFormat("dd");
		res += fmt.format(date) + " ";

		// ... avril ...
		fmt = DateTimeFormat.getFormat("MM");
		res += tabMonth[Integer.parseInt(fmt.format(date))] + " ";

		// ... 2013 à 12h22
		fmt = DateTimeFormat.getFormat("yyyy " + constants.atDate() + " HH:mm:ss");
		res += fmt.format(date);

		return res;
	}

	
	/* Handler */

	@UiHandler("button_export")
	public void onClickButtonExport(ClickEvent e){
		// TODO : la query en fonction des options
		Window.Location.replace("./csvDL?type=log&query=");
//		Window.alert("date de début (text) : " + tBox_startDate.getText() + "\ndate de début (value) : " + tBox_startDate.getValue());
	}


	@UiHandler("button_previous1")
	public void onClickPrevious1(ClickEvent e){
		if(! button_previous1.getStyleName().contains("disabled")){
			clearPanelLog();
			presenter.previousPage();
		}
	}
	@UiHandler("button_next1")
	public void onClickNext1(ClickEvent e){
		if(! button_next1.getStyleName().contains("disabled")){
			clearPanelLog();
			presenter.nextPage();
		}
	}

	@UiHandler("button_previous2")
	public void onClickPrevious2(ClickEvent e){
		if(! button_previous2.getStyleName().contains("disabled")){
			clearPanelLog();
			presenter.previousPage();
		}
	}
	@UiHandler("button_next2")
	public void onClickNext2(ClickEvent e){
		if(! button_next2.getStyleName().contains("disabled")){
			clearPanelLog();
			presenter.nextPage();
		}
	}
	
}

