package fr.janua.client.view;

import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.DivElement;
import com.google.gwt.dom.client.LIElement;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Anchor;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.RadioButton;
import com.google.gwt.user.client.ui.TextArea;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.UIObject;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.datepicker.client.DateBox;

import fr.janua.client.StringManager;
import fr.janua.client.UserManager;
import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.proxy.CommentProxy;
import fr.janua.client.proxy.MessageProxy;
import fr.janua.client.proxy.SiteProxy;
import fr.janua.client.proxy.TypeProxy;
import fr.janua.client.proxy.UserProxy;
import fr.janua.client.view.interfaces.IPageMainCouranteView;
import fr.janua.client.view.interfaces.IPageMainCouranteView.IPageMainCourantePresenter;

/**
 * 
 * @author Gilles Phan
 *
 */
public class PageMainCouranteView extends ReverseCompositeView<IPageMainCourantePresenter> implements IPageMainCouranteView {
	private static PageMainCouranteViewUiBinder uiBinder = GWT.create(PageMainCouranteViewUiBinder.class);
	interface PageMainCouranteViewUiBinder extends UiBinder<Widget, PageMainCouranteView> {}

	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);

	private MessageProxy messageForComment;	// on garde en mémoire le MessageProxy au moment de l'ouverture du modal pour ajouter un commentaire
	private MessageProxy messageForClosure;	// on garde en mémoire le MessageProxy au moment de l'ouverture du modal pour fermer le message
	private String lastSearch;				// dernière recherche effectuée
	private Set<HandlerRegistration> handlerOpenMessage;
	private Set<HandlerRegistration> handlerClosedMessage;
	
	// test
	private final static boolean TEST_MODE = false; 
	@UiField Button		button_randomText;
	
	@UiField LIElement	li_openMessage, li_closedMessage, li_addEntry, li_search;
	@UiField Label		label_error1, label_error2, label_error3, label_error4;
	@UiField Label		label_charactersRemainingForCreateMessage;
	@UiField Anchor		anchor_openMessage, anchor_closedMessage;
	@UiField DivElement	div_addMessage;
	@UiField TextBox	tBox_title;
	@UiField TextArea	tArea_description;
	@UiField FlowPanel	fPanel_openMessages, fPanel_closedMessages, fPanel_messagesFound;
	@UiField ListBox	lBox_site, lBox_type;
	@UiField FlowPanel	fPanel_overview;
	@UiField Button		button_overview, button_modify, button_valid;
	@UiField DivElement div_wrapperAllMessageReaded;
	//@UiField Anchor
	
	// pagination messages ouverts
	@UiField DivElement	div_upPaginationOpenMessage;	// up
	@UiField DivElement	div_downPaginationOpenMessage;	// down
	@UiField LIElement	li_upFirstOpenMessage,		li_upLastOpenMessage,	li_upPreviousOpenMessage,	li_upNextOpenMessage;	// up
	@UiField LIElement	li_downFirstOpenMessage,	li_downLastOpenMessage,	li_downPreviousOpenMessage,	li_downNextOpenMessage;	// down
	@UiField LIElement	li_upPageAOpenMessage,		li_upPageBOpenMessage,		li_upPageCOpenMessage,		li_upPageDOpenMessage,		li_upPageEOpenMessage;		// up
	@UiField LIElement	li_downPageAOpenMessage,	li_downPageBOpenMessage,	li_downPageCOpenMessage,	li_downPageDOpenMessage,	li_downPageEOpenMessage;	// down
	@UiField Anchor		anchor_upPageAOpenMessage,		anchor_upPageBOpenMessage,		anchor_upPageCOpenMessage,		anchor_upPageDOpenMessage,		anchor_upPageEOpenMessage;		// up
	@UiField Anchor		anchor_downPageAOpenMessage,	anchor_downPageBOpenMessage,	anchor_downPageCOpenMessage,	anchor_downPageDOpenMessage,	anchor_downPageEOpenMessage;	// down
	// pagination messages fermés
	@UiField DivElement	div_upPaginationClosedMessage;		// up
	@UiField DivElement	div_downPaginationClosedMessage;	// down
	@UiField LIElement	li_upFirstClosedMessage,	li_upPreviousClosedMessage,		li_upLastClosedMessage,		li_upNextClosedMessage;		// up
	@UiField LIElement	li_downFirstClosedMessage,	li_downPreviousClosedMessage,	li_downLastClosedMessage,	li_downNextClosedMessage;	// down
	@UiField LIElement	li_upPageAClosedMessage,		li_upPageBClosedMessage,		li_upPageCClosedMessage,		li_upPageDClosedMessage,		li_upPageEClosedMessage;	// up
	@UiField LIElement	li_downPageAClosedMessage,		li_downPageBClosedMessage,		li_downPageCClosedMessage,		li_downPageDClosedMessage,		li_downPageEClosedMessage;	// down
	@UiField Anchor		anchor_upPageAClosedMessage,	anchor_upPageBClosedMessage,	anchor_upPageCClosedMessage,	anchor_upPageDClosedMessage,	anchor_upPageEClosedMessage;	// up
	@UiField Anchor		anchor_downPageAClosedMessage,	anchor_downPageBClosedMessage,	anchor_downPageCClosedMessage,	anchor_downPageDClosedMessage,	anchor_downPageEClosedMessage;	// down
	
	
	// search
	@UiField DivElement		div_wrapperSearchCriteria;
	@UiField CheckBox		cBox_status, cBox_title, cBox_content, cBox_nbComment, cBox_location, cBox_type, cBox_author, cBox_creationDate, cBox_closureDate, cBox_creationDateBefore, cBox_creationDateAfter, cBox_closureDateBefore, cBox_closureDateAfter;
	@UiField RadioButton	rButton_stateOpen, rButton_stateClosed, rButton_0comment, rButton_oneOrMoreComment;
	@UiField TextBox		tBox_searchTitle, tBox_searchContent;
	@UiField ListBox		lBox_searchLocation, lBox_searchType, lBox_searchAuthor;
	@UiField DivElement		div_creationDate, div_closureDate; 
	@UiField Button 		button_search, button_newSearch, button_exportCsv;
	@UiField DateBox		db_creationDateBefore, db_creationDateAfter, db_closureDateBefore, db_closureDateAfter;
	@UiField Label			label_noResults;
	
	// modal info user
	@UiField Label label_modalLoginUser, label_infoFirstName, label_infoLastName, label_infoMail, label_infoGroup, label_infoState;
	
	// modal comment
	@UiField Label		label_modalCommentTitle;
	@UiField TextArea	tArea_modalComment;
	@UiField Button		button_modalCommentValidation;
	@UiField Label		label_charactersRemainingForComment;
	
	// modal closure
	@UiField Label		label_modalClosureTitle;
	@UiField TextArea	tArea_modalClosureComent;
	@UiField Button		button_modalClosureValidation1, button_modalClosureValidation2;
	@UiField Label		label_charactersRemainingForClosure;
	@UiField DivElement div_modalConfirmationClosure;
	
	public PageMainCouranteView() {
		initWidget(uiBinder.createAndBindUi(this));
	}

	// pagination
//	@UiField Label	label_numPageOpenMessage, label_numPageClosedMessage;
//	@UiField Button button_previousOpenMessage, button_previousClosedMessage, button_nextOpenMessage, button_nextClosedMessage;
	
	
	/* Override */

	@Override
	public void createView() {
		button_randomText.setVisible(TEST_MODE);
		
		handlerOpenMessage = new HashSet<HandlerRegistration>();
		handlerClosedMessage = new HashSet<HandlerRegistration>();
		
		anchor_openMessage.getElement().setAttribute("data-toggle", "tab");
		anchor_closedMessage.getElement().setAttribute("data-toggle", "tab");
		// fermer le modal au moment ou on valide le commentaire
		button_modalCommentValidation.getElement().setAttribute("data-dismiss", "modal");
		button_modalCommentValidation.getElement().setAttribute("aria-hidden", "true");
		// fermer le modal au moment ou on valide la fermeture
		button_modalClosureValidation2.getElement().setAttribute("data-dismiss", "modal");
		button_modalClosureValidation2.getElement().setAttribute("aria-hidden", "true");
		db_creationDateBefore.setFormat(new DateBox.DefaultFormat(DateTimeFormat.getFormat("dd/MM/yyyy")));
		db_creationDateAfter.setFormat(new DateBox.DefaultFormat(DateTimeFormat.getFormat("dd/MM/yyyy")));
		db_closureDateBefore.setFormat(new DateBox.DefaultFormat(DateTimeFormat.getFormat("dd/MM/yyyy")));
		db_closureDateAfter.setFormat(new DateBox.DefaultFormat(DateTimeFormat.getFormat("dd/MM/yyyy")));

	}

	@Override
	public void loadPage() {
		goToTabOpenMessage();
		
		// Ajout du message dans le title
		if(DOM.getElementById("pressCtrlForMultipleSelection") != null)
			DOM.getElementById("pressCtrlForMultipleSelection").setTitle(constants.holdCtrlToSelectMultiple());

		// on nettoie les listes de messages ouverts/fermés au moment de l'affichage de la page 
		fPanel_openMessages.clear();
		fPanel_closedMessages.clear();
		fPanel_messagesFound.clear();
		
		// on masque le bouton "J'ai lu tous les nouveaux messages"
		UIObject.setVisible(div_wrapperAllMessageReaded, false);

		messageForComment = null;
		messageForClosure = null;
		lastSearch = "";

		resetFieldSearch();
		resetFieldCreateMessage();

		
		// récupération de la liste de login de user.
		presenter.getAllLoginUser();
		
		if(!UserManager.getInstance().isViewableUser()){
			cBox_author.setEnabled(false);
		}else{
			cBox_author.setEnabled(true);
		}
		
		if(UserManager.getInstance().isLogged()){
			displayListBoxSite(UserManager.getInstance().getSitesAllowed());
			displayListBoxType(UserManager.getInstance().getTypesAllowed());
		}
		
		if(UserManager.getInstance().isCreatableMessage()){
			UIObject.setVisible(div_addMessage, true);
			label_error1.setVisible(false);
		}else{
			UIObject.setVisible(div_addMessage, false);
			label_error1.setVisible(true);
		}
		
		if(UserManager.getInstance().isViewableMessage()){
			// Si droits suffisant
			label_error2.setVisible(false);
			label_error3.setVisible(false);
			label_error4.setVisible(false);
			presenter.getNbOpenMessage();
			presenter.getNbClosedMessage();
			// affiche la pagination
			UIObject.setVisible(div_downPaginationOpenMessage, true);
			UIObject.setVisible(div_downPaginationClosedMessage, true);
		}else{
			// si non, signaler qu'on n'a pas les droits nécessaires
			label_error2.setVisible(true);
			label_error3.setVisible(true);
			label_error4.setVisible(true);
			// masque la pagination
			UIObject.setVisible(div_downPaginationOpenMessage, false);
			UIObject.setVisible(div_downPaginationClosedMessage, false);
		}
		
		
	}
	
	// Affichage des messages ouverts
	@Override
	public void displayOneMessageInOpenMessage(final MessageProxy message){
		fPanel_openMessages.add(generatePanelForMessage(message));
	}
	@Override
	public void displaySeparatorInOpenMessages() {
		// TODO : fonction à supprimer
	}

	// Affichage des messages fermés
	@Override
	public void displayOneMessageInClosedMessage(final MessageProxy message){
		fPanel_closedMessages.add(generatePanelForMessage(message));
	}
	@Override
	public void displaySeparatorInClosedMessages() {
		// TODO : fonction à supprimer 
	}
	
	// Affichage les résultats de la recherche
	@Override
	public void displayOneMessageInMessagesFound(MessageProxy m) {
		fPanel_messagesFound.add(generatePanelForMessage(m));
	}
	@Override
	public void displaySeparatorInMessagesFound() {
//		fPanel_messagesFound.add(new HTML("<div class='row-fluid'><div class='span8 offset2'></div></div>"));
	}
	@Override
	public void refreshSearch() {
		if(lastSearch != null && !"".equals(lastSearch)){
			presenter.search(lastSearch);
		}
	}
	
	// Nettoie les champs de "Ajouter une entrée"
	@Override
	public void resetFieldCreateMessage() {
		tBox_title.setValue("");
		tArea_description.setValue("");
		lBox_site.setSelectedIndex(0);
		for(int i=0; i<lBox_type.getItemCount(); i++){
			lBox_type.setItemSelected(i, false);
		}
		label_charactersRemainingForCreateMessage.setText("2500 " + constants.remainingCharacters());
		
		tBox_title.setEnabled(true);
		tArea_description.setEnabled(true);
		lBox_site.setEnabled(true);
		lBox_type.setEnabled(true);
		
		button_overview.setVisible(true);
		fPanel_overview.clear();
		button_modify.setVisible(false);
		button_valid.setVisible(false);
	}
	
	// Nettoir les messages affichés dans "Messages ouverts", "Messages fermés", ou "Recherche"
	@Override
	public void clearOpenMessage() {
		fPanel_openMessages.clear();
	}
	@Override
	public void clearClosedMessage() {
		fPanel_closedMessages.clear();
	}
	@Override
	public void clearMessagesFound(){
		fPanel_messagesFound.clear();
	}
	
	// Permet d'afficher (pas pour les mobiles) le nombre de messages ouverts/fermés dans les onglets du haut
	@Override
	public void setNbOpenMessage(Integer nbResults){
		anchor_openMessage.setHTML("<i class='icon-folder-open'></i><span class='hidden-phone'> " + constants.openMessages() + " <span class='badge badge-info'>" + nbResults + "</span></span>");
	}
	@Override
	public void setNbClosedMessage(Integer nbResults){
		anchor_closedMessage.setHTML("<i class='icon-folder-close'></i><span class='hidden-phone'> " + constants.closedMessages() + " <span class='badge badge-info'>" + nbResults + "</span></span>");
	}
	
	// Affiche la pagination
	@Override
	public void displayPaginationOpenMessage(int currentPage, int nbPage) {
		// Page précédente
		if(currentPage == 1){
			// désactivé
			li_upFirstOpenMessage.addClassName("disabled");
			li_upPreviousOpenMessage.addClassName("disabled");
			li_downFirstOpenMessage.addClassName("disabled");
			li_downPreviousOpenMessage.addClassName("disabled");
		}else{
			// activé
			li_upFirstOpenMessage.removeClassName("disabled");
			li_upPreviousOpenMessage.removeClassName("disabled");
			li_downFirstOpenMessage.removeClassName("disabled");
			li_downPreviousOpenMessage.removeClassName("disabled");
		}

		// Page suivante
		if(currentPage >= nbPage){
			// désactivé
			li_upLastOpenMessage.addClassName("disabled");
			li_upNextOpenMessage.addClassName("disabled");
			li_downLastOpenMessage.addClassName("disabled");
			li_downNextOpenMessage.addClassName("disabled");
		}else{
			// activé
			li_upLastOpenMessage.removeClassName("disabled");
			li_upNextOpenMessage.removeClassName("disabled");
			li_downLastOpenMessage.removeClassName("disabled");
			li_downNextOpenMessage.removeClassName("disabled");
		}

		// on enlève les numéros de page désactivé
		li_upPageEOpenMessage.removeClassName("disabled");
		li_upPageDOpenMessage.removeClassName("disabled");
		li_upPageCOpenMessage.removeClassName("disabled");
		li_upPageBOpenMessage.removeClassName("disabled");
		li_upPageAOpenMessage.removeClassName("disabled");
		li_downPageEOpenMessage.removeClassName("disabled");
		li_downPageDOpenMessage.removeClassName("disabled");
		li_downPageCOpenMessage.removeClassName("disabled");
		li_downPageBOpenMessage.removeClassName("disabled");
		li_downPageAOpenMessage.removeClassName("disabled");

		// on enlève les pages active
		li_upPageAOpenMessage.removeClassName("active");
		li_upPageBOpenMessage.removeClassName("active");
		li_upPageCOpenMessage.removeClassName("active");
		li_upPageDOpenMessage.removeClassName("active");
		li_upPageEOpenMessage.removeClassName("active");
		li_downPageAOpenMessage.removeClassName("active");
		li_downPageBOpenMessage.removeClassName("active");
		li_downPageCOpenMessage.removeClassName("active");
		li_downPageDOpenMessage.removeClassName("active");
		li_downPageEOpenMessage.removeClassName("active");

		if ( nbPage > 5){
			// si 6 pages ou plus
			if(currentPage<3){
				// CAS : 2 premières pages (exemple : pour la page 2/10, on affichera {1 2 3 4 5})
				// afficher curentPage-2, curentPage-1, curentPage, curentPage+1, curentPage+2
				anchor_upPageAOpenMessage.setText("1");
				anchor_upPageBOpenMessage.setText("2");
				anchor_upPageCOpenMessage.setText("3");
				anchor_upPageDOpenMessage.setText("4");
				anchor_upPageEOpenMessage.setText("5");
				anchor_downPageAOpenMessage.setText("1");
				anchor_downPageBOpenMessage.setText("2");
				anchor_downPageCOpenMessage.setText("3");
				anchor_downPageDOpenMessage.setText("4");
				anchor_downPageEOpenMessage.setText("5");
			}else if(nbPage-2 > currentPage){
				// CAS : ni les premières pages, ni les dernières (exemple : pour la page 4/10, on affichera {2 3 4 5 6})
				// afficher curentPage-2, curentPage-1, curentPage, curentPage+1, curentPage+2
				anchor_upPageEOpenMessage.setText(currentPage+2 + "");
				anchor_upPageDOpenMessage.setText(currentPage+1 + "");
				anchor_upPageCOpenMessage.setText(currentPage + "");
				anchor_upPageBOpenMessage.setText(currentPage-1 + "");
				anchor_upPageAOpenMessage.setText(currentPage-2 + "");
				anchor_downPageEOpenMessage.setText(currentPage+2 + "");
				anchor_downPageDOpenMessage.setText(currentPage+1 + "");
				anchor_downPageCOpenMessage.setText(currentPage + "");
				anchor_downPageBOpenMessage.setText(currentPage-1 + "");
				anchor_downPageAOpenMessage.setText(currentPage-2 + "");
			}else{
				// CAS : 3 dernières pages
				// afficher nbPage-4, nbPage-3, nbPage-2, nbPage1-1, nbPage
				anchor_upPageAOpenMessage.setText(nbPage-4 + "");
				anchor_upPageBOpenMessage.setText(nbPage-3 + "");
				anchor_upPageCOpenMessage.setText(nbPage-2 + "");
				anchor_upPageDOpenMessage.setText(nbPage-1 + "");
				anchor_upPageEOpenMessage.setText(nbPage   + "");
				anchor_downPageAOpenMessage.setText(nbPage-4 + "");
				anchor_downPageBOpenMessage.setText(nbPage-3 + "");
				anchor_downPageCOpenMessage.setText(nbPage-2 + "");
				anchor_downPageDOpenMessage.setText(nbPage-1 + "");
				anchor_downPageEOpenMessage.setText(nbPage   + "");
			}
		}else{
			// CAS : si moins de 5 pages
			anchor_upPageAOpenMessage.setText("1");
			anchor_upPageBOpenMessage.setText("2");
			anchor_upPageCOpenMessage.setText("3");
			anchor_upPageDOpenMessage.setText("4");
			anchor_upPageEOpenMessage.setText("5");
			anchor_downPageAOpenMessage.setText("1");
			anchor_downPageBOpenMessage.setText("2");
			anchor_downPageCOpenMessage.setText("3");
			anchor_downPageDOpenMessage.setText("4");
			anchor_downPageEOpenMessage.setText("5");
			// ajoute "disabled" pour désactivé la page dont on n'a pas accès
			switch (nbPage) {
			case 1:
				li_upPageBOpenMessage.addClassName("disabled");
				li_downPageBOpenMessage.addClassName("disabled");
			case 2:
				li_upPageCOpenMessage.addClassName("disabled");
				li_downPageCOpenMessage.addClassName("disabled");
			case 3:
				li_upPageDOpenMessage.addClassName("disabled");
				li_downPageDOpenMessage.addClassName("disabled");
			case 4:
				li_upPageEOpenMessage.addClassName("disabled");
				li_downPageEOpenMessage.addClassName("disabled");
				break;
			default:
				li_upPageAOpenMessage.addClassName("disabled");
				li_downPageAOpenMessage.addClassName("disabled");
				li_upPageBOpenMessage.addClassName("disabled");
				li_downPageBOpenMessage.addClassName("disabled");
				li_upPageCOpenMessage.addClassName("disabled");
				li_downPageCOpenMessage.addClassName("disabled");
				li_upPageDOpenMessage.addClassName("disabled");
				li_downPageDOpenMessage.addClassName("disabled");
				li_upPageEOpenMessage.addClassName("disabled");
				li_downPageEOpenMessage.addClassName("disabled");
				break;
			}

		}

		// on passe à "active" la bonne page
		if(anchor_upPageAOpenMessage.getText().equals(currentPage+"")){
			li_upPageAOpenMessage.addClassName("active");
			li_downPageAOpenMessage.addClassName("active");
		}else if(anchor_upPageBOpenMessage.getText().equals(currentPage+"")){
			li_upPageBOpenMessage.addClassName("active");
			li_downPageBOpenMessage.addClassName("active");
		}else if(anchor_upPageCOpenMessage.getText().equals(currentPage+"")){
			li_upPageCOpenMessage.addClassName("active");
			li_downPageCOpenMessage.addClassName("active");
		}else if(anchor_upPageDOpenMessage.getText().equals(currentPage+"")){
			li_upPageDOpenMessage.addClassName("active");
			li_downPageDOpenMessage.addClassName("active");
		}else if(anchor_upPageEOpenMessage.getText().equals(currentPage+"")){
			li_upPageEOpenMessage.addClassName("active");
			li_downPageEOpenMessage.addClassName("active");
		}

		for(HandlerRegistration h : handlerOpenMessage){
			h.removeHandler();
		}

		if (! li_upPageAOpenMessage.getClassName().contains("disabled")){
			handlerOpenMessage.add(anchor_upPageAOpenMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getOpenMessagePage(Integer.parseInt(anchor_upPageAOpenMessage.getText()));
				}
			}));
			handlerOpenMessage.add(anchor_downPageAOpenMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getOpenMessagePage(Integer.parseInt(anchor_downPageAOpenMessage.getText()));
				}
			}));
		}
		if (! li_upPageBOpenMessage.getClassName().contains("disabled")){
			handlerOpenMessage.add(anchor_upPageBOpenMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getOpenMessagePage(Integer.parseInt(anchor_upPageBOpenMessage.getText()));
				}
			}));
			handlerOpenMessage.add(anchor_downPageBOpenMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getOpenMessagePage(Integer.parseInt(anchor_downPageBOpenMessage.getText()));
				}
			}));
		}
		if (! li_upPageCOpenMessage.getClassName().contains("disabled")){
			handlerOpenMessage.add(anchor_upPageCOpenMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getOpenMessagePage(Integer.parseInt(anchor_upPageCOpenMessage.getText()));
				}
			}));
			handlerOpenMessage.add(anchor_downPageCOpenMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getOpenMessagePage(Integer.parseInt(anchor_downPageCOpenMessage.getText()));
				}
			}));
		}
		if (! li_upPageDOpenMessage.getClassName().contains("disabled")){
			handlerOpenMessage.add(anchor_upPageDOpenMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getOpenMessagePage(Integer.parseInt(anchor_upPageDOpenMessage.getText()));
				}
			}));
			handlerOpenMessage.add(anchor_downPageDOpenMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getOpenMessagePage(Integer.parseInt(anchor_downPageDOpenMessage.getText()));
				}
			}));
		}
		if (! li_upPageEOpenMessage.getClassName().contains("disabled")){
			handlerOpenMessage.add(anchor_upPageEOpenMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getOpenMessagePage(Integer.parseInt(anchor_upPageEOpenMessage.getText()));
				}
			}));
			handlerOpenMessage.add(anchor_downPageEOpenMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getOpenMessagePage(Integer.parseInt(anchor_downPageEOpenMessage.getText()));
				}
			}));
		}

		
		
		
		
		
		// OLD !!
		
		
//		label_numPageOpenMessage.setVisible(true);
//		button_previousOpenMessage.setVisible(true);
//		button_nextOpenMessage.setVisible(true);
//		
//		label_numPageOpenMessage.setText(currentPage + "/" + ((nbPage>0)?nbPage:1));
//		
//		button_previousOpenMessage.removeStyleName("disabled");
//		button_nextOpenMessage.removeStyleName("disabled");
//
//		// si 1ère page ou dernière page : désactiver le bouton 1ère page ou dernière page
//		if(currentPage<=1){
//			button_previousOpenMessage.addStyleName("disabled");
//		}
//		if(currentPage>=nbPage){
//			button_nextOpenMessage.addStyleName("disabled");
//		}
	}
	@Override
	public void displayPaginationClosedMessage(int currentPage, int nbPage) {
		// Page précédente
		if(currentPage == 1){
			// désactivé
			li_upFirstClosedMessage.addClassName("disabled");
			li_upPreviousClosedMessage.addClassName("disabled");
			li_downFirstClosedMessage.addClassName("disabled");
			li_downPreviousClosedMessage.addClassName("disabled");
		}else{
			// activé
			li_upFirstClosedMessage.removeClassName("disabled");
			li_upPreviousClosedMessage.removeClassName("disabled");
			li_downFirstClosedMessage.removeClassName("disabled");
			li_downPreviousClosedMessage.removeClassName("disabled");
		}

		// Page suivante
		if(currentPage >= nbPage){
			// désactivé
			li_upLastClosedMessage.addClassName("disabled");
			li_upNextClosedMessage.addClassName("disabled");
			li_downLastClosedMessage.addClassName("disabled");
			li_downNextClosedMessage.addClassName("disabled");
		}else{
			// activé
			li_upLastClosedMessage.removeClassName("disabled");
			li_upNextClosedMessage.removeClassName("disabled");
			li_downLastClosedMessage.removeClassName("disabled");
			li_downNextClosedMessage.removeClassName("disabled");
		}

		// on enlève les numéros de page désactivé
		li_upPageEClosedMessage.removeClassName("disabled");
		li_upPageDClosedMessage.removeClassName("disabled");
		li_upPageCClosedMessage.removeClassName("disabled");
		li_upPageBClosedMessage.removeClassName("disabled");
		li_upPageAClosedMessage.removeClassName("disabled");
		li_downPageEClosedMessage.removeClassName("disabled");
		li_downPageDClosedMessage.removeClassName("disabled");
		li_downPageCClosedMessage.removeClassName("disabled");
		li_downPageBClosedMessage.removeClassName("disabled");
		li_downPageAClosedMessage.removeClassName("disabled");

		// on enlève les pages active
		li_upPageAClosedMessage.removeClassName("active");
		li_upPageBClosedMessage.removeClassName("active");
		li_upPageCClosedMessage.removeClassName("active");
		li_upPageDClosedMessage.removeClassName("active");
		li_upPageEClosedMessage.removeClassName("active");
		li_downPageAClosedMessage.removeClassName("active");
		li_downPageBClosedMessage.removeClassName("active");
		li_downPageCClosedMessage.removeClassName("active");
		li_downPageDClosedMessage.removeClassName("active");
		li_downPageEClosedMessage.removeClassName("active");

		if ( nbPage > 5){
			// si 6 pages ou plus
			if(currentPage<3){
				// CAS : 2 premières pages (exemple : pour la page 2/10, on affichera {1 2 3 4 5})
				// afficher curentPage-2, curentPage-1, curentPage, curentPage+1, curentPage+2
				anchor_upPageAClosedMessage.setText("1");
				anchor_upPageBClosedMessage.setText("2");
				anchor_upPageCClosedMessage.setText("3");
				anchor_upPageDClosedMessage.setText("4");
				anchor_upPageEClosedMessage.setText("5");
				anchor_downPageAClosedMessage.setText("1");
				anchor_downPageBClosedMessage.setText("2");
				anchor_downPageCClosedMessage.setText("3");
				anchor_downPageDClosedMessage.setText("4");
				anchor_downPageEClosedMessage.setText("5");
			}else if(nbPage-2 > currentPage){
				// CAS : ni les premières pages, ni les dernières (exemple : pour la page 4/10, on affichera {2 3 4 5 6})
				// afficher curentPage-2, curentPage-1, curentPage, curentPage+1, curentPage+2
				anchor_upPageEClosedMessage.setText(currentPage+2 + "");
				anchor_upPageDClosedMessage.setText(currentPage+1 + "");
				anchor_upPageCClosedMessage.setText(currentPage + "");
				anchor_upPageBClosedMessage.setText(currentPage-1 + "");
				anchor_upPageAClosedMessage.setText(currentPage-2 + "");
				anchor_downPageEClosedMessage.setText(currentPage+2 + "");
				anchor_downPageDClosedMessage.setText(currentPage+1 + "");
				anchor_downPageCClosedMessage.setText(currentPage + "");
				anchor_downPageBClosedMessage.setText(currentPage-1 + "");
				anchor_downPageAClosedMessage.setText(currentPage-2 + "");
			}else{
				// CAS : 3 dernières pages
				// afficher nbPage-4, nbPage-3, nbPage-2, nbPage1-1, nbPage
				anchor_upPageAClosedMessage.setText(nbPage-4 + "");
				anchor_upPageBClosedMessage.setText(nbPage-3 + "");
				anchor_upPageCClosedMessage.setText(nbPage-2 + "");
				anchor_upPageDClosedMessage.setText(nbPage-1 + "");
				anchor_upPageEClosedMessage.setText(nbPage   + "");
				anchor_downPageAClosedMessage.setText(nbPage-4 + "");
				anchor_downPageBClosedMessage.setText(nbPage-3 + "");
				anchor_downPageCClosedMessage.setText(nbPage-2 + "");
				anchor_downPageDClosedMessage.setText(nbPage-1 + "");
				anchor_downPageEClosedMessage.setText(nbPage   + "");
			}
		}else{
			// CAS : si moins de 5 pages
			anchor_upPageAClosedMessage.setText("1");
			anchor_upPageBClosedMessage.setText("2");
			anchor_upPageCClosedMessage.setText("3");
			anchor_upPageDClosedMessage.setText("4");
			anchor_upPageEClosedMessage.setText("5");
			anchor_downPageAClosedMessage.setText("1");
			anchor_downPageBClosedMessage.setText("2");
			anchor_downPageCClosedMessage.setText("3");
			anchor_downPageDClosedMessage.setText("4");
			anchor_downPageEClosedMessage.setText("5");
			// ajoute "disabled" pour désactivé la page dont on n'a pas accès
			switch (nbPage) {
			case 1:
				li_upPageBClosedMessage.addClassName("disabled");
				li_downPageBClosedMessage.addClassName("disabled");
			case 2:
				li_upPageCClosedMessage.addClassName("disabled");
				li_downPageCClosedMessage.addClassName("disabled");
			case 3:
				li_upPageDClosedMessage.addClassName("disabled");
				li_downPageDClosedMessage.addClassName("disabled");
			case 4:
				li_upPageEClosedMessage.addClassName("disabled");
				li_downPageEClosedMessage.addClassName("disabled");
				break;
			default:
				li_upPageAOpenMessage.addClassName("disabled");
				li_downPageAOpenMessage.addClassName("disabled");
				li_upPageBOpenMessage.addClassName("disabled");
				li_downPageBOpenMessage.addClassName("disabled");
				li_upPageCOpenMessage.addClassName("disabled");
				li_downPageCOpenMessage.addClassName("disabled");
				li_upPageDOpenMessage.addClassName("disabled");
				li_downPageDOpenMessage.addClassName("disabled");
				li_upPageEOpenMessage.addClassName("disabled");
				li_downPageEOpenMessage.addClassName("disabled");
				break;
			}

		}

		// on passe à "active" la bonne page
		if(anchor_upPageAClosedMessage.getText().equals(currentPage+"")){
			li_upPageAClosedMessage.addClassName("active");
			li_downPageAClosedMessage.addClassName("active");
		}else if(anchor_upPageBClosedMessage.getText().equals(currentPage+"")){
			li_upPageBClosedMessage.addClassName("active");
			li_downPageBClosedMessage.addClassName("active");
		}else if(anchor_upPageCClosedMessage.getText().equals(currentPage+"")){
			li_upPageCClosedMessage.addClassName("active");
			li_downPageCClosedMessage.addClassName("active");
		}else if(anchor_upPageDClosedMessage.getText().equals(currentPage+"")){
			li_upPageDClosedMessage.addClassName("active");
			li_downPageDClosedMessage.addClassName("active");
		}else if(anchor_upPageEClosedMessage.getText().equals(currentPage+"")){
			li_upPageEClosedMessage.addClassName("active");
			li_downPageEClosedMessage.addClassName("active");
		}

		for(HandlerRegistration h : handlerClosedMessage){
			h.removeHandler();
		}

		if (! li_upPageAClosedMessage.getClassName().contains("disabled")){
			handlerClosedMessage.add(anchor_upPageAClosedMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getClosedMessagePage(Integer.parseInt(anchor_upPageAClosedMessage.getText()));
				}
			}));
			handlerClosedMessage.add(anchor_downPageAClosedMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getClosedMessagePage(Integer.parseInt(anchor_downPageAClosedMessage.getText()));
				}
			}));
		}
		if (! li_upPageBClosedMessage.getClassName().contains("disabled")){
			handlerClosedMessage.add(anchor_upPageBClosedMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getClosedMessagePage(Integer.parseInt(anchor_upPageBClosedMessage.getText()));
				}
			}));
			handlerClosedMessage.add(anchor_downPageBClosedMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getClosedMessagePage(Integer.parseInt(anchor_downPageBClosedMessage.getText()));
				}
			}));
		}
		if (! li_upPageCClosedMessage.getClassName().contains("disabled")){
			handlerClosedMessage.add(anchor_upPageCClosedMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getClosedMessagePage(Integer.parseInt(anchor_upPageCClosedMessage.getText()));
				}
			}));
			handlerClosedMessage.add(anchor_downPageCClosedMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getClosedMessagePage(Integer.parseInt(anchor_downPageCClosedMessage.getText()));
				}
			}));
		}
		if (! li_upPageDClosedMessage.getClassName().contains("disabled")){
			handlerClosedMessage.add(anchor_upPageDClosedMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getClosedMessagePage(Integer.parseInt(anchor_upPageDClosedMessage.getText()));
				}
			}));
			handlerClosedMessage.add(anchor_downPageDClosedMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getClosedMessagePage(Integer.parseInt(anchor_downPageDClosedMessage.getText()));
				}
			}));
		}
		if (! li_upPageEClosedMessage.getClassName().contains("disabled")){
			handlerClosedMessage.add(anchor_upPageEClosedMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getClosedMessagePage(Integer.parseInt(anchor_upPageEClosedMessage.getText()));
				}
			}));
			handlerClosedMessage.add(anchor_downPageEClosedMessage.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getClosedMessagePage(Integer.parseInt(anchor_downPageEClosedMessage.getText()));
				}
			}));
		}

// OLD
//		label_numPageClosedMessage.setVisible(true);
//		button_previousClosedMessage.setVisible(true);
//		button_nextClosedMessage.setVisible(true);
//		
//		label_numPageClosedMessage.setText(currentPageClosedMessage + "/" + ((nbPage>0)?nbPage:1));
//		
//		button_previousClosedMessage.removeStyleName("disabled");
//		button_nextClosedMessage.removeStyleName("disabled");
//
//		// si 1ère page ou dernière page : désactiver le bouton 1ère page ou dernière page
//		if(currentPageClosedMessage<=1){
//			button_previousClosedMessage.addStyleName("disabled");
//		}
//		if(currentPageClosedMessage>=nbPage){
//			button_nextClosedMessage.addStyleName("disabled");
//		}
	}
	
	@Override
	public void setListLogin(List<String> listLogins) {
		lBox_searchAuthor.clear();
		for(String login : listLogins)
			lBox_searchAuthor.addItem(login);
	}
	
	@Override
	public void goToTabOpenMessage() {
		// Gestion des onglets
		if(DOM.getElementById("lA") != null)
			DOM.getElementById("lA").removeClassName("active");
		if(DOM.getElementById("lB") != null)
			DOM.getElementById("lB").addClassName("active");
		if(DOM.getElementById("lC") != null)
			DOM.getElementById("lC").removeClassName("active");
		if(DOM.getElementById("lD") != null)
			DOM.getElementById("lD").removeClassName("active");

		// Gestion des contenus
		li_openMessage.addClassName("active");
		li_closedMessage.removeClassName("active");
		li_addEntry.removeClassName("active");
		li_search.removeClassName("active");
	}
	
	@Override
	public void displayProfileInModal(UserProxy user) {
		label_modalLoginUser.setText(user.getLogin());
		label_infoFirstName.setText(user.getFirstName());
		label_infoLastName.setText(user.getLastName());
		label_infoMail.setText(user.getMail());
		label_infoGroup.setText(user.getGroup().getName());
		label_infoState.setText((user.isEnabled()) ? constants.enabled() : constants.disabled());
	}

	@Override
	public void noResultsFound() {
		label_noResults.setVisible(true);
	}
	
	
	
	/* Fonctions */
	
	/**
	 * Génère un FlowPanel à partir du message passé en paramètre.
	 * Ce FlowPanel est un aperçu du message, avec tous les champs du MessageProxy affiché.
	 * Il pourra être intégré dans les messages ouverts, messages fermés, ou comme résultat de la recherche.
	 * @return
	 */
	private FlowPanel generatePanelForMessage(final MessageProxy message){
		/*
		 * Le code suivant se déroule comme suit :
		 * 1 - Titre
		 * 2 - Description message
		 * 3 - Saisi par
		 * [si commentaires]
		 *    4 - Toogle commentaires (plier/déplier)
		 * [fin si commentaires]
		 * [boucle]
		 *    5.a - Commentaire
		 *    5.b - "Commmenté par ..." / "Par ..."
		 * [fin boucle]
		 * [si message fermé]
		 *    6 - fermé par
		 * 7 - Site
		 * 8 - Types
		 * 9 - Footer
		 *    [si message ouvert]
		 *    	 9.a - Ajouter type
		 *    	 9.b - Séparateur
		 *       9.c - Ajouter commentaire ("commenter")
		 *       9.d - Séparateur
		 *       9.e - Fermer message
		 *    [si message fermé]
		 *       9.a - Télécharger le pdf
		 */
		final int nbComment = message.getComments().size();
		boolean isNewMessage = UserManager.getInstance().getLastActivity() < message.getLastChange();

		FlowPanel fPanelBlocMessage = new FlowPanel();
		fPanelBlocMessage.addStyleName("bloc-info span8 offset2");
		
		// 1 - Titre
		String newMessage = "";
		if(isNewMessage){
			UIObject.setVisible(div_wrapperAllMessageReaded, true);
			newMessage = "<div class='label label-important' style='background:OrangeRed;'>new</div>";
		}
		
		String cssTitle = "";
		if(message.getStatus()==1)
			cssTitle = "title-blue";
		else
			cssTitle = "title-orange";
		fPanelBlocMessage.add(new HTML(
				"<div class='row-fluid'>" +
				"    <div class='span12 " + cssTitle + " gradient'>" +
				"        " + newMessage + " " + StringManager.encodeHtml(message.getTitle()) + " <small class='pull-right padding-right-10'><em>(" + longDateToString(message.getCreationDate()) + ")</em> <em class='label label-inverse'>#" + message.getId() + "</em></small>" +
				"    </div>" +
			 	"</div>"));

		// 2 - Description message
		fPanelBlocMessage.add(new HTML("<div class='row-fluid'><div class='span12 description-message'>" + StringManager.encodeHtml(message.getDescription()) + "</div></div>"));
		
		// 3 - Saisi par
		HTML htmlSaisiPar = new HTML("<div class='row-fluid'><div class='span12 description-message text-right'><small>" + constants.seizedBy() + " <em><a href='#modalInfoUser' role='button' data-toggle='modal'>" + message.getNameCreator() + "</a> (" + message.getGroupCreator() + ")</em> " + "</small></div></div>");
		htmlSaisiPar.addStyleName("separator-dashed");
		if(UserManager.getInstance().isViewableUser()){
			htmlSaisiPar.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					clearModalInfoUser();
					presenter.getProfile(message.getNameCreator());
				}
			});
		}else{
			htmlSaisiPar.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					noAccessRightsModalInfoUser(message.getNameCreator());
				}
			});
		}
		fPanelBlocMessage.add(htmlSaisiPar);
		
		final FlowPanel fPanelComments = new FlowPanel();
		fPanelComments.setVisible(isNewMessage);	// dépli les commentaire si c'est un nouveau message
		// 4 - Toogle commentaires
		
		// affiche un bouton "-" si nouveau message (car le commentaires seront dépliés)
		// affiche un bouton "+" si message  ordinaire (car le commentaires seront replés)
		final HTML htmlToogleComments = (isNewMessage) ? 
				new HTML("<div class='span2 padding-top-10 padding-left-10'><span title='" + constants.toggleComments() + " (" + nbComment + ")' class='pointer ellipsis'>-</span></div>") :
				new HTML("<div class='span2 padding-top-10 padding-left-10'><span title='" + constants.toggleComments() + " (" + nbComment + ")' class='pointer ellipsis'>+</span></div>");
		htmlToogleComments.addStyleName("span2");
		htmlToogleComments.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				if(fPanelComments.isVisible()){
					fPanelComments.setVisible(false);
					htmlToogleComments.setHTML("<div class='span2 padding-top-10 padding-left-10'><span title='" + constants.toggleComments() + " (" + nbComment + ")' class='pointer ellipsis'>+</span></div>");
				}else{
					fPanelComments.setVisible(true);
					htmlToogleComments.setHTML("<div class='span2 padding-top-10 padding-left-10'><span title='" + constants.toggleComments() + " (" + nbComment + ")' class='pointer ellipsis'>-</span></div>");
				}
			}
		});
		
		
		Set<CommentProxy> copyOfComments = message.getComments(); // plus un lien qu'une copie
		while(copyOfComments.size() > 0){
			// 5.a - Commentaires
			CommentProxy c = null;
			for(CommentProxy commentProxy : copyOfComments){
				if(c==null){
					c = commentProxy;
					continue;
				}else if(c.getDate() > commentProxy.getDate()){
					c = commentProxy;
				}
			}
			
			TypeProxy t = c.getType();
			String by;	// "Added by " or "By "
			if(t == null){
				// ce Comment est un commentaire écrit par un utilisateur
				fPanelComments.add(new HTML("<div class='row-fluid'><div class='span12 description-message'>" + StringManager.encodeHtml(c.getDescription()) + "</div></div>"));
				by = constants.seizedBy();
			}else{
				// ce Comment est un ajout de type
				String typeAdded = constants.typeAdded() + " <span class='label' style='background:" + t.getBgColor() + ";color:" + t.getFgColor() + ";'>" + t.getName() + "</span>";
				fPanelComments.add(new HTML("<div class='row-fluid'><div class='span12 description-message'>" + typeAdded + "</div></div>"));
				by = constants.by();
			}
			// 5.b - "Commmenté par ..." / "Par ..."
			HTML htmlCommentedBy = new HTML("<div class='row-fluid'><div class='span12 description-message text-right'><small>" + by + "<em> <a href='#modalInfoUser' role='button' data-toggle='modal'>" + c.getAuthor() + "</a> (" + c.getGroupName() + ")</em> " + constants.onDate() + " " + longDateToString(c.getDate()) + "</small></div></div>");
			// si droits suffisant, il peut cliquer pour voir les détails d'un User
			final String authorComment = c.getAuthor();
			if(UserManager.getInstance().isViewableUser()){
				htmlCommentedBy.addClickHandler(new ClickHandler() {
					@Override
					public void onClick(ClickEvent event) {
						clearModalInfoUser();
						presenter.getProfile(authorComment);
					}
				});
			}else{
				htmlCommentedBy.addClickHandler(new ClickHandler() {
					@Override
					public void onClick(ClickEvent event) {
						noAccessRightsModalInfoUser(message.getNameCreator());
					}
				});
			}
			fPanelComments.add(htmlCommentedBy);
			htmlCommentedBy.addStyleName("separator-dashed");
			
			copyOfComments.remove(c);
		}
		fPanelBlocMessage.add(fPanelComments);
		
		if(message.getStatus() == 0){
			// 6 - fermé par
			if(message.getCommentClosure().length() == 0)
				fPanelBlocMessage.add(new HTML("<div class='row-fluid'><div class='span12 description-message'>" + constants.noComment() + "</div></div>"));
			else
				fPanelBlocMessage.add(new HTML("<div class='row-fluid'><div class='span12 description-message'>" + message.getCommentClosure() + "</div></div>"));
			
			HTML htmlClosedBy = new HTML("<div class='row-fluid'><div class='span12 description-message text-right'><small>" + constants.closedBy() + " <em><a href='#modalInfoUser' role='button' data-toggle='modal'>" + message.getNameCloser() + "</a> (" + message.getGroupCloser() + ")</em> " + constants.onDate() + " " + longDateToString(message.getClosureDate()) + "</small></div></div>");
			htmlClosedBy.addStyleName("separator-dashed");
			if(UserManager.getInstance().isViewableUser()){
				htmlClosedBy.addClickHandler(new ClickHandler() {
					@Override
					public void onClick(ClickEvent event) {
						clearModalInfoUser();
						presenter.getProfile(message.getNameCloser());
					}
				});
			}else{
				htmlClosedBy.addClickHandler(new ClickHandler() {
					@Override
					public void onClick(ClickEvent event) {
						noAccessRightsModalInfoUser(message.getNameCloser());
					}
				});
			}
			fPanelBlocMessage.add(htmlClosedBy);
		}

		// 7 - ToggleComment - Site
		FlowPanel fPanelWrapperSiteToggle = new FlowPanel();
		fPanelWrapperSiteToggle.addStyleName("row-fluid");
		HTML htmlSite = new HTML("<div class='text-right description-message'><small>" + constants.location() + " : " + message.getSite().getName() + "</small></div>");
		htmlSite.addStyleName("span10");
		if(nbComment==0)
			htmlToogleComments.setHTML("");
		fPanelWrapperSiteToggle.add(htmlToogleComments);
		fPanelWrapperSiteToggle.add(htmlSite);
		fPanelBlocMessage.add(fPanelWrapperSiteToggle);
		
		// 8 - Type
		String allTypes = "";
		for(TypeProxy typeProxy : message.getTypes()){
			allTypes += "<span style='background:" + typeProxy.getBgColor() + ";color:" + typeProxy.getFgColor() + ";border-radius:4px;padding:2px;'>" + typeProxy.getName() + "</span>, ";
		}
		// on enlève la dernière virgule pour faire propre
		if (allTypes.length() != 0)
			allTypes = allTypes.substring(0, allTypes.length()-2);
		HTML htmlType = new HTML("<div class='row-fluid'><div class='span12 text-right description-message'><small>" + constants.type() + " : " + allTypes + "</small></div></div>");
		fPanelBlocMessage.add(htmlType);
		
		// si editableMessage : alors il peut ajouter un type
		final FlowPanel fPanelAddType = new FlowPanel();
		if(UserManager.getInstance().isEditableMessage()){
			fPanelAddType.addStyleName("text-center");
			fPanelAddType.setVisible(false);
			
			final ListBox lBoxAddType = new ListBox();
			final Button buttonValidAddType = new Button(constants.validate());
			for(TypeProxy type : UserManager.getInstance().getTypesAllowed())
				if(!message.getTypes().contains(type))
					lBoxAddType.addItem(type.getName());
			
			lBoxAddType.addStyleName("inline");
			buttonValidAddType.addStyleName("btn btn-info inline");
			buttonValidAddType.getElement().setAttribute("style", "margin: 0px 0px 10px 25px;");	// remettre le bouton à niveau par rapport au ListBox de types
			
			lBoxAddType.setVisible(true);
			buttonValidAddType.setVisible(true);
			
			buttonValidAddType.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.addType(message, lBoxAddType.getItemText(lBoxAddType.getSelectedIndex()));
				}
			});
			
			fPanelAddType.add(lBoxAddType);
			fPanelAddType.add(buttonValidAddType);
			
			fPanelBlocMessage.add(fPanelAddType);
		}


		// 9 - Footer
		FlowPanel footerBloc = new FlowPanel();
		footerBloc.setStyleName("text-center");
		
		if(message.getStatus() == 1){
			if(UserManager.getInstance().isEditableMessage()){
				// 9.a - Ajouter un type
				Anchor anchorAddType = new Anchor(constants.addType());
				//anchorComment.setHref("#");
				anchorAddType.setStyleName("inline");
				anchorAddType.addClickHandler(new ClickHandler() {
					@Override
					public void onClick(ClickEvent event) {
						if(fPanelAddType.isVisible())
							fPanelAddType.setVisible(false);
						else
							fPanelAddType.setVisible(true);
					}
				});
				footerBloc.add(anchorAddType);
				
				// 9.b - Séparateur
				Label verticalSeparator2 = new Label(" | ");
				verticalSeparator2.setStyleName("inline");
				footerBloc.add(verticalSeparator2);
			}
			// 9.c - Ajouter un commentaire
			Anchor anchorComment = new Anchor(constants.comment());
			anchorComment.setHref("#modalComment");
			anchorComment.setStyleName("inline");
			anchorComment.getElement().setAttribute("role", "button");      // attribut pour ouvrir le modal (bootstrap)
			anchorComment.getElement().setAttribute("data-toggle", "modal");// attribut pour ouvrir le modal (bootstrap)
			anchorComment.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					clearModalComment(); // on efface le modal avant de l'afficher
					label_modalCommentTitle.setText("#" + message.getId() + " - " + message.getTitle());
					messageForComment = message;
				}
			});
			footerBloc.add(anchorComment);
	
			// 9.d - Séparateur
			Label verticalSeparator3 = new Label(" | ");
			verticalSeparator3.setStyleName("inline");
			footerBloc.add(verticalSeparator3);
			
			// 9.e - Fermer message (en fait, on ouvre le modal qui s'occupera de fermer le message)
			Anchor anchorClose = new Anchor(constants.close());
			anchorClose.setHref("#modalClosure");
			anchorClose.setStyleName("inline");
			anchorClose.getElement().setAttribute("role", "button");
			anchorClose.getElement().setAttribute("data-toggle", "modal");
			anchorClose.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					clearModalClosure();
					label_modalClosureTitle.setText("#" + message.getId() + " - " + message.getTitle());
					messageForClosure = message;
					UIObject.setVisible(div_modalConfirmationClosure, false);
				}
			});
			footerBloc.add(anchorClose);

		}else{
			// 9.a - Télécharger le pdf
			Anchor anchorGeneratePdf = new Anchor(constants.downloadPDF());
			anchorGeneratePdf.setStyleName("inline");
			anchorGeneratePdf.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					String login = UserManager.getInstance().getLogin();
					String password = UserManager.getInstance().getPassword();
					String idMessage = message.getId() + "";
					String namePDF = message.getId() + "-" + message.getClosureDate() + ".pdf";
					Window.Location.replace("./pdfDL?"
							+ "login="+login
							+ "&password="+password
							+ "&namePDF="+namePDF		// ne surtout pas mettre celui là en fin de requête, car un servlet empêche l'accès aux fichiers *.pdf
							+ "&idMessage="+idMessage);
				}
			});
			footerBloc.add(anchorGeneratePdf);
		}
		fPanelBlocMessage.add(footerBloc);

		// wrapper pour ajouter un row fluid
		FlowPanel fPanelWrapperBlocMessage = new FlowPanel();
		fPanelWrapperBlocMessage.setStyleName("row-fluid");
		fPanelWrapperBlocMessage.add(fPanelBlocMessage);
		
		return fPanelWrapperBlocMessage;
	}
	
	/**
	 * Remplis le ListBox de type avec les Type donc le groupe de l'utilisateur a accès.
	 * @param setTypes
	 */
	public void displayListBoxType(Set<TypeProxy> setTypes) {
		lBox_type.clear();
		lBox_searchType.clear();
		// remplissage du ListBox avec les nouvelles données récupérés
		for(TypeProxy type : setTypes){
			if(type.isEnabled()){
				lBox_type.addItem(type.getName());
				lBox_searchType.addItem(type.getName());
			}
		}
	}
	/**
	 * Remplis le ListBox de site avec les Site donc le groupe de l'utilisateur a accès.
	 * @param setSite
	 */
	public void displayListBoxSite(Set<SiteProxy> setSite) {
		lBox_site.clear();
		lBox_searchLocation.clear();
		lBox_site.addItem(constants.selectASite(), "");
		// remplissage du ListBox avec les nouvelles données récupérés
		for(SiteProxy site : setSite){
			if(site.isEnabled()){
				lBox_site.addItem(site.getName());
				lBox_searchLocation.addItem(site.getName());
			}
		}
	}
	
	/**
	 * Efface toutes les données saisies dans le modal "Info user"
	 */
	private void clearModalInfoUser(){
		label_modalLoginUser.setText(constants.loading());
		label_infoFirstName.setText(constants.loading());
		label_infoLastName.setText(constants.loading());
		label_infoMail.setText(constants.loading());
		label_infoGroup.setText(constants.loading());
		label_infoState.setText(constants.loading());
	}
	/**
	 * Efface toutes les données saisies dans le modal "Commentaire"
	 */
	private void clearModalComment(){
		label_modalCommentTitle.setText("---");
		tArea_modalComment.setText("");
		label_charactersRemainingForComment.setText("2500 " + constants.remainingCharacters());
	}
	/**
	 * Efface toutes les données saisies dans le modal "Fermer le message"
	 */
	private void clearModalClosure(){
		label_modalClosureTitle.setText("---");
		tArea_modalClosureComent.setText("");
		label_charactersRemainingForClosure.setText("2500 " + constants.remainingCharacters());
	}
	/**
	 * Affiche dans le modal "Info user" qu'il n'a pas les droits pour voir les info sur l'utilisateur.
	 * @param user
	 */
	private void noAccessRightsModalInfoUser(String user){
		label_modalLoginUser.setText(user);
		label_infoFirstName.setText(constants.insufficientAccessRights());
		label_infoLastName.setText(constants.insufficientAccessRights());
		label_infoMail.setText(constants.insufficientAccessRights());
		label_infoGroup.setText(constants.insufficientAccessRights());
		label_infoState.setText(constants.insufficientAccessRights());
	}
	/**
	 * Convertit une date de type long vers une String formaté pour donner la date correspondante.
	 * Format : "30 avril 2013 à 11:16:25"
	 * @param time
	 * @return
	 */
	private String longDateToString(long time){
		String[] tabMonth = {"error Date", constants.january(), constants.febuary(), constants.march(), constants.april(), constants.may(), constants.june(), constants.july(), constants.august(), constants.september(), constants.october(), constants.november(), constants.december()};
		String res = "";
		Date date = new Date(time);

		// 30 ...
		DateTimeFormat fmt = DateTimeFormat.getFormat("dd");
		res += fmt.format(date) + " ";
		
		// ... avril ...
		fmt = DateTimeFormat.getFormat("MM");
		res += tabMonth[Integer.parseInt(fmt.format(date))] + " ";
		
		// ... 2013 à 12h22
		fmt = DateTimeFormat.getFormat("yyyy " + constants.atDate() + " HH:mm:ss");
		res += fmt.format(date);
		
		return res;
	}
	
	/**
	 * Affiche un aperçu d'un Message ouvert. L'utilisateur pourra ensuite valider ou modifier le message.
	 * Note : On est obligé d'afficher l'aperçu avant de le valider.
	 */
	private void generateOverview(){
		// On efface le précédent aperçu
		fPanel_overview.clear();

		// Un Widget FlowPanel qui contiendra la totalité de l'aperçu
		FlowPanel blocMessage = new FlowPanel();
		blocMessage.addStyleName("bloc-info span8 offset2");

		String[] tabMonth = {"error Date", constants.january(), constants.febuary(), constants.march(), constants.april(), constants.may(), constants.june(), constants.july(), constants.august(), constants.september(), constants.october(), constants.november(), constants.december()};
		Date d = new Date();																	// La date du jour
		String date = DateTimeFormat.getFormat( "d" ).format(d) + " ";							// Le numéro du mois
		date += tabMonth[Integer.parseInt(DateTimeFormat.getFormat( "MM" ).format(d))] + " ";	// Le mois
		date += DateTimeFormat.getFormat( "yyyy " + constants.atDate() + " HH:mm:ss" ).format(d);						// L'année et l'heure

		// Un Widget HTML pour contenir le titre
		blocMessage.add(new HTML("<div class='row-fluid'><div class='span12 title-blue'>" + StringManager.encodeHtml(tBox_title.getText()) + "<small class='pull-right padding-right-10'><em>(" + date + ")</em> <em class='label label-inverse'>#0</em></small></div></div>"));

		// Un Widget HTML pour contenir la description du message
		blocMessage.add(new HTML("<div class='row-fluid'><div class='span12 description-message'>" + StringManager.encodeHtml(tArea_description.getText()) + "</div></div>"));

		// Un Widget HTML pour contenir l'auteur
		HTML htmlSaisiPar = new HTML("<div class='row-fluid'><div class='span12 description-message text-right'><small>" + constants.seizedBy() + " <em>" + UserManager.getInstance().getLogin() + "</em></small></div></div>");
		htmlSaisiPar.addStyleName("separator-dashed");
		blocMessage.add(htmlSaisiPar);
		
		
		// Site
		HTML htmlSite = new HTML("<div class='row-fluid'><div class='span12 text-right description-message'><small>" + constants.location() + " : " + lBox_site.getItemText(lBox_site.getSelectedIndex()) + "</small></div></div>");
		blocMessage.add(htmlSite);
		
		// Type
		String allTypes = "";
		for(int i=0; i<lBox_type.getItemCount(); i++){
			if(lBox_type.isItemSelected(i)){
				String typeSelected = lBox_type.getItemText(i);
				for(TypeProxy type : UserManager.getInstance().getTypesAllowed()){
					if(type.getName().equals(typeSelected))
						allTypes += "<span style='background:" + type.getBgColor() + ";color:" + type.getFgColor() + ";border-radius:4px;padding:2px;'>" + type.getName() + "</span>, ";
				}
			}
		}
		// on enlève la dernière virgule pour faire propre
		if (allTypes.length() != 0)
			allTypes = allTypes.substring(0, allTypes.length()-2);
		HTML htmlType = new HTML("<div class='row-fluid'><div class='span12 text-right description-message'><small>" + constants.type() + " : " + allTypes + "</small></div></div>");
		blocMessage.add(htmlType);

		fPanel_overview.add(blocMessage);
	}
	
	/**
	 * Remet les champs par défaut, et masque tous les champs de saisis.
	 */
	private void resetFieldSearch(){
		cBox_status.setValue(false);
		cBox_title.setValue(false);
		cBox_content.setValue(false);
		cBox_nbComment.setValue(false);
		cBox_location.setValue(false);
		cBox_type.setValue(false);
		cBox_author.setValue(false);
		cBox_creationDate.setValue(false);
		cBox_closureDate.setValue(false);
		cBox_creationDateBefore.setValue(false);
		cBox_creationDateAfter.setValue(false);
		cBox_closureDateBefore.setValue(false);
		cBox_closureDateAfter.setValue(false);
		
		label_noResults.setVisible(false);
		rButton_stateOpen.setVisible(false);
		rButton_stateClosed.setVisible(false);
		tBox_searchTitle.setVisible(false);
		tBox_searchContent.setVisible(false);
		rButton_0comment.setVisible(false);
		rButton_oneOrMoreComment.setVisible(false);
		lBox_searchLocation.setVisible(false);
		lBox_searchType.setVisible(false);
		lBox_searchAuthor.setVisible(false);
		UIObject.setVisible(div_creationDate, false);
		UIObject.setVisible(div_closureDate, false);
		
		rButton_stateClosed.setValue(true);
		rButton_stateOpen.setValue(false);
		tBox_searchTitle.setText("");
		tBox_searchContent.setText("");
		rButton_0comment.setValue(true);
		rButton_oneOrMoreComment.setValue(false);
		lBox_searchLocation.setSelectedIndex(0);
		for(int i=0; i<lBox_searchType.getItemCount(); i++)
			lBox_searchType.setItemSelected(i, false);
		lBox_searchAuthor.setSelectedIndex(0);
		db_creationDateBefore.setValue(null);
		db_creationDateAfter.setValue(null);
		db_closureDateBefore.setValue(null);
		db_closureDateAfter.setValue(null);
		UIObject.setVisible(div_wrapperSearchCriteria, true);
		button_search.setVisible(true);
		button_newSearch.setVisible(false);
		button_exportCsv.setVisible(false);
	}

	/**
	 * Convertit une Date en un Long.
	 * @param date
	 * @return
	 */
	private long stringToLongDate(Date date){
//		DateTimeFormat fmt = DateTimeFormat.getFormat("yyyy-MM-dd");
//		Date d = fmt.parse(date);
		return date.getTime();
	}
	
	/**
	 * Ajoute un jour à la date passé en paramètre (date en long).
	 * @param date
	 * @return
	 */
	private long incrDay(long date){
		return date + (1000*60*60*24);
	}
	

	/* Handlers */
	
	@UiHandler("button_allMessagesReaded")
	public void onClickButtonAllMessagesReaded(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage()){
			// on masque le bouton "J'ai lu tous les nouveaux messages"
			UIObject.setVisible(div_wrapperAllMessageReaded, false);
			// et on met à jour la date de dernière connexion
			presenter.updateLastActivity();
		}
	}
	
	@UiHandler("button_overview")
	public void onClickButtonOverviewMessage(ClickEvent e){
		// booléen pour dire si on a sélectionné un type ou pas
		boolean isTypeSelected = false;
		for(int i=0; i<lBox_type.getItemCount(); i++){
			if(lBox_type.isItemSelected(i)){
				isTypeSelected = true;
				break;
			}
		}
		
		// check si les champs sont bien remplis
		if(tBox_title.getValue().equals("")){
			Window.alert(constants.pleaseEnterTitle());
		}else if(tBox_title.getText().length()>255){
			Window.alert(constants.pleaseEnterShorterTitle());
		}else if(tArea_description.getValue().equals("")){
			Window.alert(constants.pleaseEnterDescription());
		}else if(tArea_description.getText().length()>2500){
			Window.alert(constants.pleaseEnterShorterDescription());
		}else if(lBox_site.getSelectedIndex() == 0){
			Window.alert(constants.pleaseSelectSite());
		}else if(!isTypeSelected){
			Window.alert(constants.pleaseSelectTypes());
		}else{
			// on désactive les champs de saisi
			tBox_title.setEnabled(false);
			tArea_description.setEnabled(false);
			lBox_site.setEnabled(false);
			lBox_type.setEnabled(false);
			// on masque le bouton "Aperçu"
			button_overview.setVisible(false);
			
			// on affiche les boutons "Modifier" pour revenir en arrière et "Valider" pour confirmer la création du message
			button_modify.setVisible(true);
			button_valid.setVisible(true);
			
			// on génuère un aperçu du message
			generateOverview();
		}
	}
	
	@UiHandler("button_modify")
	public void onClickButtonModifyMessage(ClickEvent e){
		tBox_title.setEnabled(true);
		tArea_description.setEnabled(true);
		lBox_site.setEnabled(true);
		lBox_type.setEnabled(true);
		
		button_overview.setVisible(true);
		
		fPanel_overview.clear();
		
		button_modify.setVisible(false);
		button_valid.setVisible(false);
		
		
	}
	
	@UiHandler("button_valid")
	public void onClickButtonValidMessage(ClickEvent e){
		// booléen pour dire si on a sélectionné un type ou pas
		boolean isTypeSelected = false;
		for(int i=0; i<lBox_type.getItemCount(); i++){
			if(lBox_type.isItemSelected(i)){
				isTypeSelected = true;
				break;
			}
		}
				
		// check si les champs sont bien remplis
		if(tBox_title.getValue().equals("")){
			Window.alert(constants.pleaseEnterTitle());
		}else if(tBox_title.getText().length()>255){
			Window.alert(constants.pleaseEnterShorterTitle());
		}else if(tArea_description.getValue().equals("")){
			Window.alert(constants.pleaseEnterDescription());
		}else if(tArea_description.getText().length()>2500){
			Window.alert(constants.pleaseEnterShorterDescription());
		}else if(lBox_site.getSelectedIndex() == 0){
			Window.alert(constants.pleaseSelectSite());
		}else if(!isTypeSelected){
			Window.alert(constants.pleaseSelectTypes());
		}else{
			/*
			 * Si les champs sont remplis, alors on commence envoie tout au presenter
			 * pour qu'il crée le proxy et qu'il l'enregistre dans la BDD.
			 */
			String title = tBox_title.getText();
			String description = tArea_description.getText();
			String siteSelected = lBox_site.getValue(lBox_site.getSelectedIndex());
			SiteProxy site = null;
			Set<TypeProxy> types = new HashSet<TypeProxy>();

			for(SiteProxy siteProxy : UserManager.getInstance().getSitesAllowed()){
				if(siteSelected.equals(siteProxy.getName()))
					site = siteProxy;
			}
			
			// pour chaque TypeProxy de la ListeBox
			for(int i=0; i<lBox_type.getItemCount(); i++){
				// S'il est sélectionné (i.e. coché) ...
				if(lBox_type.isItemSelected(i)){
					String currentItemValue = lBox_type.getValue(i);
					// ... le rechercher dans notre ArrayList<OtherTypeProxy>...
					for(TypeProxy currentTypeProxy : UserManager.getInstance().getTypesAllowed()){
						if(currentTypeProxy.getName().equals(currentItemValue)){
							// ... et l'ajouter à notre Set de Type.
							types.add(currentTypeProxy);
						}
					}
				}
			}
			
			presenter.createMessage(title, description, site, types);
		}
	}

	
	// Validation modal comment/closure
	@UiHandler("button_modalCommentValidation")
	public void onClickButtonValidModalComment(ClickEvent e){
		if(tArea_modalComment.getText().length()>2500){
			Window.alert(constants.pleaseEnterShorterComment());
		}else{
			String comment = tArea_modalComment.getText();
			// check commentaire !null
			if(comment.equals("")){
				Window.alert(constants.pleaseEnterComment());
				return;
			}else{
				presenter.addComment(messageForComment, comment);
				clearModalComment();
			}
		}
	}
	@UiHandler("button_modalClosureValidation1")
	public void onClickButtonValidModalClosure1(ClickEvent e){
		UIObject.setVisible(div_modalConfirmationClosure, true);
	}
	@UiHandler("button_modalClosureValidation2")
	public void onClickButtonValidModalClosure2(ClickEvent e){
		if(tArea_modalClosureComent.getText().length()>2500){
			Window.alert(constants.pleaseEnterShorterComment());
		}else{
			presenter.closeMessage(messageForClosure, tArea_modalClosureComent.getText());
			clearModalClosure();
		}
	}
	
	
	// Pagination up   Message Ouverts (page next, page previous)
	@UiHandler("anchor_upPreviousOpenMessage")
	public void onClickButtonUpPreviousOpenMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_upPreviousOpenMessage.getClassName().contains("disabled"))
			presenter.previousPageOpenMessage();
	}
	@UiHandler("anchor_upNextOpenMessage")
	public void onClickButtonUpNextOpenMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_upNextOpenMessage.getClassName().contains("disabled"))
			presenter.nextPageOpenMessage();
	}
	@UiHandler("anchor_upFirstOpenMessage")
	public void onClickButtonUpFirstOpenMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_upFirstOpenMessage.getClassName().contains("disabled"))
			presenter.getOpenMessagePage(1);
	}
	@UiHandler("anchor_upLastOpenMessage")
	public void onClickButtonUpLastOpenMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_upLastOpenMessage.getClassName().contains("disabled"))
			presenter.lastPageOpenMessage();
	}
	// Pagination down Message Ouverts (page next, page previous)
	@UiHandler("anchor_downPreviousOpenMessage")
	public void onClickButtonDownPreviousOpenMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_downPreviousOpenMessage.getClassName().contains("disabled"))
			presenter.previousPageOpenMessage();
	}
	@UiHandler("anchor_downNextOpenMessage")
	public void onClickButtonDownNextOpenMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_downNextOpenMessage.getClassName().contains("disabled"))
			presenter.nextPageOpenMessage();
	}
	@UiHandler("anchor_downFirstOpenMessage")
	public void onClickButtonDownFirstOpenMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_downFirstOpenMessage.getClassName().contains("disabled"))
			presenter.getOpenMessagePage(1);
	}
	@UiHandler("anchor_downLastOpenMessage")
	public void onClickButtonDownLastOpenMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_downLastOpenMessage.getClassName().contains("disabled"))
			presenter.lastPageOpenMessage();
	}
	

	// Pagination up   Message Fermés (page next, page previous)
	@UiHandler("anchor_upPreviousClosedMessage")
	public void onClickButtonUpPreviousClosedMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_upPreviousClosedMessage.getClassName().contains("disabled"))
			presenter.previousPageClosedMessage();
	}
	@UiHandler("anchor_upNextClosedMessage")
	public void onClickButtonUpNextClosedMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_upNextClosedMessage.getClassName().contains("disabled"))
			presenter.nextPageClosedMessage();
	}
	@UiHandler("anchor_upFirstClosedMessage")
	public void onClickButtonUpFirstClosedMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_upFirstClosedMessage.getClassName().contains("disabled"))
			presenter.getClosedMessagePage(1);
	}
	@UiHandler("anchor_upLastClosedMessage")
	public void onClickButtonUpLastClosedMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_upLastClosedMessage.getClassName().contains("disabled"))
			presenter.lastPageClosedMessage();
	}
	// Pagination down Message Fermés (page next, page previous)
	@UiHandler("anchor_downPreviousClosedMessage")
	public void onClickButtonDownPreviousClosedMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_downPreviousClosedMessage.getClassName().contains("disabled"))
			presenter.previousPageClosedMessage();
	}
	@UiHandler("anchor_downNextClosedMessage")
	public void onClickButtonDownNextClosedMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_downNextClosedMessage.getClassName().contains("disabled"))
			presenter.nextPageClosedMessage();
	}
	@UiHandler("anchor_downFirstClosedMessage")
	public void onClickButtonDownFirstClosedMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_downFirstClosedMessage.getClassName().contains("disabled"))
			presenter.getClosedMessagePage(1);
	}
	@UiHandler("anchor_downLastClosedMessage")
	public void onClickButtonDownLastClosedMessage(ClickEvent e){
		if(UserManager.getInstance().isViewableMessage() && !li_downLastClosedMessage.getClassName().contains("disabled"))
			presenter.lastPageClosedMessage();
	}
	
	
	// 2500 caractères restants : créer un message
	@UiHandler("tArea_description")
	public void onKeyupDescription(KeyUpEvent e){
		int nbCharacterRemainging = 2500 - tArea_description.getText().length();
		if(nbCharacterRemainging>1)
			label_charactersRemainingForCreateMessage.setText(nbCharacterRemainging + " " + constants.remainingCharacters());
		else
			label_charactersRemainingForCreateMessage.setText(nbCharacterRemainging + " " + constants.remainingCharacters());
	}
	@UiHandler("tArea_description")
	public void onChangeDescription(ChangeEvent e){
		int nbCharacterRemainging = 2500 - tArea_description.getText().length();
		if(nbCharacterRemainging>1)
			label_charactersRemainingForCreateMessage.setText(nbCharacterRemainging + " " + constants.remainingCharacters());
		else
			label_charactersRemainingForCreateMessage.setText(nbCharacterRemainging + " " + constants.remainingCharacters());
	}
	// 2500 caractères restants : commenter un message
	@UiHandler("tArea_modalComment")
	public void onKeyupComment(KeyUpEvent e){
		int nbCharacterRemainging = 2500 - tArea_modalComment.getText().length();
		if(nbCharacterRemainging>1)
			label_charactersRemainingForComment.setText(nbCharacterRemainging + " " + constants.remainingCharacters());
		else
			label_charactersRemainingForComment.setText(nbCharacterRemainging + " " + constants.remainingCharacters());
	}
	@UiHandler("tArea_modalComment")
	public void onChangeComment(ChangeEvent e){
		int nbCharacterRemainging = 2500 - tArea_modalComment.getText().length();
		if(nbCharacterRemainging>1)
			label_charactersRemainingForComment.setText(nbCharacterRemainging + " " + constants.remainingCharacters());
		else
			label_charactersRemainingForComment.setText(nbCharacterRemainging + " " + constants.remainingCharacters());
	}
	// 2500 caractères restants : fermer un message
	@UiHandler("tArea_modalClosureComent")
	public void onKeyupClosure(KeyUpEvent e){
		int nbCharacterRemainging = 2500 - tArea_modalClosureComent.getText().length();
		if(nbCharacterRemainging>1)
			label_charactersRemainingForClosure.setText(nbCharacterRemainging + " " + constants.remainingCharacters());
		else
			label_charactersRemainingForClosure.setText(nbCharacterRemainging + " " + constants.remainingCharacters());
	}
	@UiHandler("tArea_modalClosureComent")
	public void onChangeClosure(ChangeEvent e){
		int nbCharacterRemainging = 2500 - tArea_modalClosureComent.getText().length();
		if(nbCharacterRemainging>1)
			label_charactersRemainingForClosure.setText(nbCharacterRemainging + " " + constants.remainingCharacters());
		else
			label_charactersRemainingForClosure.setText(nbCharacterRemainging + " " + constants.remainingCharacters());
	}

	@UiHandler("button_exportCsv")
	public void onClickExportCSV(ClickEvent e){
		Window.Location.replace("./csvDL?type=message&query=" + lastSearch);
	}
	
	
	

	// CheckBox search
	@UiHandler("button_search")
	public void onClickButtonSearch(ClickEvent e){
		// on masque les champs de saisis
		UIObject.setVisible(div_wrapperSearchCriteria, false);
		// on masque le bouton "Rechercher"
		button_search.setVisible(false);
		// et on affiche led boutond "nouvelle recherche" et "exporter en csv"
		button_newSearch.setVisible(true);
		button_exportCsv.setVisible(true);
		
		// TODO : première partie à mettre côté serveur
		String query = "SELECT DISTINCT m FROM Message m JOIN m.site site JOIN m.types types " + "WHERE ";

		// trier les messages qui concordent avec le site de l'utilisateur
		query += "site.name in (";
		for(SiteProxy s : UserManager.getInstance().getSitesAllowed()){
			query += "'" + s.getName() + "',";
		}
		query = query.substring(0, query.length()-1);
		// trier les messages qui concordent avec les types de l'utilisateur
		query += ") AND types.name in (";
		for(TypeProxy t : UserManager.getInstance().getTypesAllowed()){
			query += "'" + t.getName() + "',";
		}
		query = query.substring(0, query.length()-1);
		query += ") AND ";
		
		//  messages ouverts ou fermés
		if(cBox_status.getValue()){
			if(rButton_stateOpen.getValue())
				query += "m.status=1 AND ";
			else
				query += "m.status=0 AND ";
		}
		
		// contenu dans le titre
		if(cBox_title.getValue()){
			query += "m.title like '%" + tBox_searchTitle.getText().replace(" ", "%") + "%' AND ";
		}
		
		// contenu dans le corps du message, ou dans le commentaire de fermeture
		if(cBox_content.getValue()){
			query += "(m.description like '%" + tBox_searchContent.getText().replace(" ", "%") + "%' OR " +
					 "m.commentClosure like '%" + tBox_searchContent.getText().replace(" ", "%") + "%') AND ";
		}
		
		// 0 ou plusieurs commentaires
		if(cBox_nbComment.getValue()){
			if(rButton_0comment.getValue())
				query += "m.comments IS EMPTY AND ";
			else
				query += "size(m.comments)>0 AND ";
		}
		
		// site des messages
		if(cBox_location.getValue()){
			query += "site.name like '" + lBox_searchLocation.getItemText(lBox_searchLocation.getSelectedIndex()) + "' AND ";
		}
		
		// types des messages
		if(cBox_type.getValue()){
			query += "types.name in (";
			for(int i=0; i<lBox_searchType.getItemCount(); i++){
				if(lBox_searchType.isItemSelected(i)){
					query += "'" + lBox_searchType.getItemText(i) + "',";
				}
			}
			query = query.substring(0, query.length()-1);
			query += ") AND ";
		}
		
		// auteur du message
		if(cBox_author.getValue()){
			query += "m.nameCreator like '" + lBox_searchAuthor.getItemText(lBox_searchAuthor.getSelectedIndex()) + "' AND ";
		}
		
		if(cBox_creationDate.getValue()){
			if(cBox_creationDateBefore.getValue())
				query += "m.creationDate<" + incrDay(stringToLongDate(db_creationDateBefore.getValue())) + "  AND ";
			if(cBox_creationDateAfter.getValue())
				query += "m.creationDate>" + stringToLongDate(db_creationDateAfter.getValue()) + "  AND ";
		}
		if(cBox_closureDate.getValue()){
			if(cBox_closureDateBefore.getValue())
				query += "m.closureDate<" + incrDay(stringToLongDate(db_closureDateBefore.getValue())) + "  AND ";
			if(cBox_closureDateAfter.getValue())
				query += "m.closureDate>" + stringToLongDate(db_closureDateAfter.getValue()) + "  AND ";
		}
		
		// enlève le dernier "AND "
		if(query.endsWith(" AND "))
			query = query.substring(0, query.length()-4);
		query += "ORDER BY m.lastChange DESC";
		
		lastSearch = query;
//		Window.alert(query);
		
		presenter.search(query);
	}

	@UiHandler("button_newSearch")
	public void onClickButtonNewSearch(ClickEvent e){
		clearMessagesFound();
		resetFieldSearch();
	}
	
	@UiHandler("cBox_status")
	public void onCheckStatus(ClickEvent e){
		rButton_stateOpen.setVisible(cBox_status.getValue());
		rButton_stateClosed.setVisible(cBox_status.getValue());
	}
	
	@UiHandler("cBox_title")
	public void onCheckTitle(ClickEvent e){
		tBox_searchTitle.setVisible(cBox_title.getValue());
	}
	
	@UiHandler("cBox_content")
	public void onCheckContent(ClickEvent e){
		tBox_searchContent.setVisible(cBox_content.getValue());
	}
	
	@UiHandler("cBox_nbComment")
	public void onCheckNbComment(ClickEvent e){
		rButton_0comment.setVisible(cBox_nbComment.getValue());
		rButton_oneOrMoreComment.setVisible(cBox_nbComment.getValue());
	}
	
	@UiHandler("cBox_location")
	public void onCheckLocation(ClickEvent e){
		lBox_searchLocation.setVisible(cBox_location.getValue());
	}
	
	@UiHandler("cBox_type")
	public void onCheckType(ClickEvent e){
		lBox_searchType.setVisible(cBox_type.getValue());
	}
	
	@UiHandler("cBox_author")
	public void onCheckAuthor(ClickEvent e){
		lBox_searchAuthor.setVisible(cBox_author.getValue());
	}
	
	@UiHandler("cBox_creationDate")
	public void onCheckCreationDate(ClickEvent e){
		UIObject.setVisible(div_creationDate, cBox_creationDate.getValue());
	}
	
	@UiHandler("cBox_closureDate")
	public void onCheckClosureDate(ClickEvent e){
		UIObject.setVisible(div_closureDate, cBox_closureDate.getValue());
		
		// si check du CheckBox "date de fermeture"...
		if(cBox_closureDate.getValue()){
			// ... alors cocher automatiquement "statut des messages : fermés" 
			cBox_status.setValue(true);
			rButton_stateOpen.setVisible(cBox_status.getValue());
			rButton_stateClosed.setVisible(cBox_status.getValue());
			rButton_stateClosed.setValue(true);
		}
	}

	

	
	// test
	@UiHandler("button_randomText")
	public void onClickRandomText(ClickEvent e){
		String[] tabRandomText = {	"Toc toc, anyone hear me ?",
									"The chalk establishs the tin.",
									"The eager brass supervises the grip.",
									"When does the tested idea discharge the swim?",
									"The kiss debugs the base.",
									"The cloth check ins the wide society.",
									"The smart shade files the silver.",
									"The drink starts the fat fight.",
									"The ragged flight devises the organisation.",
									"The prose lobbys the cloudy sex.",
									"The way boughts the lavish existence.",
									"The night writes the cough.",
									"The superb hour combines the sugar.",
									"The motion co-operates the nauseating event.",
									"The credit reports the back.",
									"The reading substitutes the deadpan pain.",
									"The quixotic system set goalss the care.",
									"The loutish value convinces the oil.",
									"The cloudy control proposes the thought.",
									"The wind conserves the graceful copper.",
									"The ashamed scale surpasss the reading.",
									"The name rates the argument.",
									"The scale studys the staking twist.",
									"The chance individualizes the succinct grain.",
									"The amount outlines the manager.",
									"The sign updates the magnificent mark.",
									"The familiar development unveils the memory.",
									"The account measures the base.",
									"The prose introduces the cook.",
									"The wine builds the petite name.",
									"The messy rate initiates the tendency.",
									};
		int randomText = (int)(Math.random() * (tabRandomText.length-1));
		tArea_description.setText(tabRandomText[randomText]);
		
		int randomSite = (int)(Math.random() * (lBox_site.getItemCount()-1)) +1;
		lBox_site.setSelectedIndex(randomSite);
		
		lBox_type.setSelectedIndex(-1);
		for(int i=0; i<lBox_type.getItemCount()/2; i++){
			int randomType = (int)(Math.random() * (lBox_type.getItemCount()));
			lBox_type.setItemSelected(randomType, true);
		}
		
		tBox_title.setText("Hello world, random #" + randomText + ", #" + randomSite);
	}

	
	

}
