package fr.janua.client.view.interfaces;

import java.util.List;
import java.util.Set;

import com.google.gwt.user.client.ui.IsWidget;
import com.mvp4g.client.view.LazyView;

import fr.janua.client.proxy.GroupProxy;
import fr.janua.client.proxy.SiteProxy;
import fr.janua.client.proxy.UserProxy;

public interface IPageUserView extends IsWidget , LazyView {
	public interface IPageUserPresenter{
		/**
		 * Création d'un User.
		 * @param login
		 * @param firstName
		 * @param lastName
		 * @param mail
		 * @param groupName
		 * @param password
		 * @param sitesAllowed 
		 * @param enabled
		 */
		void createUser(String login, String firstName, String lastName, String mail, String groupName, String password, Set<SiteProxy> sitesAllowed, boolean enabled);

		/**
		 * Met à jour le User.
		 * @param login
		 * @param firstName
		 * @param lastName
		 * @param mail
		 * @param siteAllowed 
		 * @param groupName
		 * @param enabled
		 */
		void updateUser(String login, String firstName, String lastName, String mail, Set<SiteProxy> siteAllowed, String groupName, boolean enabled);

		/**
		 * Récupère la liste de tous les Role, PUIS lance la récupération de tous les profils.
		 */
		void getAllSitesAllRoleGetAllUser();

		/**
		 * Lance le chargement des User. Cette étape se fait en deux phases :
		 * <ul>
		 * 	<li>Première requête qui demande combien de User contient la BDD, afin de pouvoir générer une pagination.</li>
		 * 	<li>Seconde requête qui va charger les n premiers User pour le système de pagination.</li>
		 * </ul>
		 */
		void generatePagination();

		/**
		 * Demande au presenter d'aller à la page suivante.
		 */
		void previousPage();

		/**
		 * Demande au presenter d'aller à la page précédente.
		 */
		void nextPage();

		/**
		 * Demande au presenter d'aller à la page <code>numPage</code>.
		 * @param numPage
		 */
		void getUserPage(int numPage);

		/**
		 * Demande au presenter d'aller à la dernière page.
		 */
		void lastPage();

		/**
		 * Demande à la base de donnée si le login est disponible ou non.
		 * @param text
		 */
		void checkAvailability(String text);

		/**
		 * Met à jour le mot de passe.
		 * @param login
		 * @param password
		 */
		void updatePassword(String login, String password);


		
	}

	/**
	 * Fonction appelée à chaque fois que l'on va charger la page.
	 * @param name
	 */
	void loadPage();

	/**
	 * Affiche un User
	 * @param user
	 */
	void displayOneUser(UserProxy user);

	/**
	 * Définit la liste des Role dans la partie View. Permet de remplir les ListBox. 
	 * @param listGroup
	 */
	void setListGroup(List<GroupProxy> listGroup);

	/**
	 * Affiche la pagination.
	 * @param currentPage
	 * @param nbPage le nombre de page à générer dans la pagination.
	 */
	void displayPagination(int currentPage, int nbPage);

	/**
	 * Efface tous les profils affiché.
	 */
	void clearPanelUser();

	/**
	 * Signale si le login est disponible ou pas.
	 * @param available
	 */
	void loginAvailable(boolean available);

	/**
	 * Enregistre la liste de site courante.
	 * @param listSites
	 */
	void setListSite(List<SiteProxy> listSites);

}
