package fr.janua.server;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Set;

import com.google.web.bindery.requestfactory.server.RequestFactoryServlet;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Font;
import com.itextpdf.text.FontFactory;
import com.itextpdf.text.Image;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;

import fr.janua.server.entity.Comment;
import fr.janua.server.entity.Message;
import fr.janua.server.entity.Type;

/**
 * Classe Singleton permettant de gérer la création de pdf en français.
 * @author gilles
 *
 */
public final class PdfManager{
	private static volatile PdfManager instance = null;
	private static String fullPathPDF;
	private static String fullPathCSV;
	
	
	
	private PdfManager() {
		fullPathPDF = RequestFactoryServlet.getThreadLocalServletContext().getRealPath(File.separator + "pdf") + File.separator;
		fullPathCSV = RequestFactoryServlet.getThreadLocalServletContext().getRealPath(File.separator + "csv") + File.separator;
	}


	/**
	 * Premier appel au singleton.
	 * @return
	 */
	public final static PdfManager getInstance() {
		//		if (null == instance) {
		//			// enregistre dans : ./pdf/
		//			// ....getRealPath(File.separator + "pdf" + File.separator); => ne marche pas quand on déploie le war, il ne voit pas le dernire "/"
		//			instance = new PdfManager();
		//			fullPath = RequestFactoryServlet.getThreadLocalServletContext().getRealPath(File.separator + "pdf") + File.separator;
		//			System.out.println("fullPath for PDF : " + fullPath);
		//		}
		//		return instance;
				
		
		
        if (PdfManager.instance == null) {
           synchronized(PdfManager.class) {
             if (PdfManager.instance == null) {
            	 PdfManager.instance = new PdfManager();
             }
           }
        }
        return PdfManager.instance;
	}
	
	

	/**
	 * @return the fullPath
	 */
	public String getFullPathPDF() {
		if(fullPathPDF==null){
			fullPathPDF = RequestFactoryServlet.getThreadLocalServletContext().getRealPath(File.separator + "pdf") + File.separator;
		}
		System.out.println("fullPath for PDF : " + fullPathPDF);
		return fullPathPDF ;
	}
	public String getFullPathCSV() {
		if(fullPathCSV==null){
			fullPathCSV = RequestFactoryServlet.getThreadLocalServletContext().getRealPath(File.separator + "csv") + File.separator;
		}
		System.out.println("fullPath for CSV : " + fullPathCSV);
		return fullPathCSV;
	}


	public Boolean createPdf(Message message){
		//String fullPath = RequestFactoryServlet.getThreadLocalServletContext().getRealPath(File.separator + "pdf" + File.separator);
		String namePdf = message.getId() + "-" + message.getClosureDate() + ".pdf";

		long time = System.currentTimeMillis();
		try{
			Document document = new Document();
			PdfWriter.getInstance(document, new FileOutputStream(fullPathPDF + namePdf));
			document.open();

			// MetaData
			document.addTitle("Message #" + message.getId() + "-" + message.getTitle());
			document.addSubject("Rapport d'intervention");
			document.addKeywords("Java, PDF, iText, GWT, Jaguards, Janua");
			document.addAuthor(message.getNameCreator());
			document.addCreator("jaguards");

			// Title
			addTitle(document, message);

			// Contenu
			addContent(document, message);
			document.close();
		}catch(DocumentException e){
			e.printStackTrace();
			return false;
		}catch(IOException e){
			e.printStackTrace();
			return false;
		}

		System.out.println("Création du pdf dans " + fullPathPDF + namePdf + " effectuée en " + ((System.currentTimeMillis()-time)/1000.0) + "sec !");

		return true;
	}

	/**
	 * Génère le titre du document.
	 * @param document
	 * @param message
	 * @throws DocumentException
	 * @throws IOException
	 */
	private void addTitle(Document document, Message message) throws DocumentException, IOException{
		Paragraph title = new Paragraph();

		String imagePath = fullPathPDF + "logo.png";
		Image logoJanua = null;
		try{
			logoJanua = Image.getInstance(imagePath);
			title.add(logoJanua);
		}catch(MalformedURLException e){
			e.printStackTrace();
		}
		
		String types = "";
		for(Type t : message.getTypes()){
			types += t.getName() + ", ";
		}
		types = types.substring(0, types.length()-2);
		
		title.add(new Paragraph("Titre : " + message.getTitle()));
		title.add(new Paragraph("Message n° " + message.getId()));
		title.add(new Paragraph("Site : " + message.getSite().getName()));
		title.add(new Paragraph("Type : " + types));
		title.add(new Paragraph("PDF généré le " + longToTime(System.currentTimeMillis())));

		addEmptyLine(title, 4);

		document.add(title);
	}

	/**
	 * Génère le corps du document.
	 * @param document
	 * @param message
	 * @throws DocumentException
	 */
	private void addContent(Document document, Message message) throws DocumentException {
		Font fontBlue = FontFactory.getFont(FontFactory.TIMES_ROMAN, 12);
		fontBlue.setColor(new BaseColor(67, 0, 153));
		Font fontBlack = FontFactory.getFont(FontFactory.TIMES_ROMAN, 12);
		fontBlack.setColor(BaseColor.BLACK);

		// Creation d'une PdfPTable avec 1 colonnes
		PdfPTable table = new PdfPTable(1);
		// largeur : 100%
		table.setWidthPercentage(100);

		// description
		Phrase description = new Phrase(message.getDescription().replace("<br/>", "\n"));
		Phrase saisiPar = new Phrase("Saisi par " + message.getNameCreator() + "(" + message.getGroupCreator() + ") le " + longToTime(message.getCreationDate()) + ".");

		description.setFont(fontBlack);
		saisiPar.setFont(fontBlue);

		PdfPCell cellDescription = new PdfPCell();
		cellDescription.addElement(description);
		cellDescription.addElement(saisiPar);

		table.addCell(cellDescription);

		// commentaires
		Set<Comment> copyOfComments = message.getComments(); // plus un lien qu'une copie
		while(copyOfComments.size() > 0){
			// recherche de la date la plus ancienne
			Comment c = null;
			for(Comment comment : copyOfComments){
				if(c==null){
					c = comment;
					continue;
				}else if(c.getDate() > comment.getDate()){
					c = comment;
				}
			}
			
			Phrase comment;
			Phrase commentedBy;
			if(c.getType() == null){
				// un commentaire
				comment = new Phrase(c.getDescription().replace("<br/>", "\n"));
				commentedBy = new Phrase("Commenté par " + c.getAuthor() + " (" + c.getGroupName() + ") le " + longToTime(c.getDate()) + ".");
			}else{
				// un ajout de type
				comment = new Phrase("Type ajouté : " + c.getType().getName());
				commentedBy = new Phrase("Par " + c.getAuthor() + " (" + c.getGroupName() + ") le " + longToTime(c.getDate()) + ".");
			}
			comment.setFont(fontBlack);
			commentedBy.setFont(fontBlue);

			PdfPCell cellComment = new PdfPCell();
			cellComment.addElement(comment);
			cellComment.addElement(commentedBy);

			table.addCell(cellComment);

			// on enlève le commentaire du HashSet
			copyOfComments.remove(c);
		}

		// commentaire de fermeture
		PdfPCell cellCommentClosure = new PdfPCell();

		Phrase commentClosure = null;
		if(message.getCommentClosure() != null){ 
			commentClosure = new Phrase(message.getCommentClosure().replace("<br/>", "\n"));
		}else{
			commentClosure =  new Phrase("Aucun commentaire.");
		}
		Phrase closedBy = new Phrase("Fermé par " + message.getNameCloser() + "(" + message.getGroupCloser() + ") le " + longToTime(message.getClosureDate()) + ".");

		commentClosure.setFont(fontBlack);
		closedBy.setFont(fontBlue);

		cellCommentClosure.addElement(commentClosure);		
		cellCommentClosure.addElement(closedBy);

		table.addCell(cellCommentClosure);

		// Ajout d'un espace entre la PdfPTable et l'élément précédent.
		table.setSpacingBefore(15f);
		document.add(table);
	}

	/**
	 * Ajoute <code>number</code> sauts de ligne.
	 * @param paragraph
	 * @param number
	 */
	private void addEmptyLine(Paragraph paragraph, int number) {
		for (int i = 0; i < number; i++) {
			paragraph.add(new Paragraph(" "));
		}
	}



	/**
	 * Convertit une date de long en String.
	 * @param time
	 * @return
	 */
	private String longToTime(long time){
		String[] tabMonth = {"error Date", "janvier", "février", "mars", "avril", "mai", "juin", "juillet", "août", "septembre", "octobre", "novembre", "décembre"};
		String res = "";
		Date date = new Date(time);
		// 30 ...
		DateFormat fmt = new SimpleDateFormat("dd");
		res += fmt.format(date) + " ";
		// ... avril ...
		fmt = new SimpleDateFormat("MM");
		res += tabMonth[Integer.parseInt(fmt.format(date))] + " ";
		// ... 2013 à 12h22
		fmt = new SimpleDateFormat("yyyy à HH:mm:ss");
		res += fmt.format(date);
		return res;
	}


}
