package fr.janua.server.dao.object;

import java.util.List;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import fr.janua.server.dao.DAO;
import fr.janua.server.utils.HibernateUtils;
import fr.janua.shared.Log;

public class LogDAO extends DAO<Log>{

	public LogDAO() {
		super();
	}

	@Override
	public boolean create(Log log) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.save(log);
			tx.commit();
		} catch (Exception e) {
			e.printStackTrace();
			if (tx != null && session.isOpen())
				tx.rollback();
			return false;
		}finally{
			session.close();
		}
		return true;
	}

	@Override
	public boolean delete(Log log) {
		// TODO : à coder
		return false;
	}

	@Override
	public boolean update(Log log) {
		System.err.println("Utilisation de update Log : interdite.");
		return false;
	}

	@Override
	public Log findById(int id) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT log FROM Log log WHERE log.id = :id");
		query.setParameter("id", id);
		Log log = (Log) query.uniqueResult();
		session.close();
		
		return log;
	}

	/**
	 * Récupère tous les Log de la base de données. Peut être long.
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<Log> findAllLog() {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT log FROM Log log ORDER BY log.date DESC");
		List<Log> listLog = query.list();
		session.close();
		
		return listLog;
	}
	
	/**
	 * Récupère une liste de n Log.
	 * @param startIndex
	 * @param nbLog
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<Log> findLog(int startIndex, int nbLog) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT log FROM Log log ORDER BY log.date DESC");
		
		query.setFirstResult(startIndex);
		query.setMaxResults(nbLog);
		
		List<Log> listLog = query.list();
		session.close();
		
		return listLog;
	}
	
	/**
	 * Récupère une liste de n Log, dont l'auteur est "login".
	 * @param login
	 * @param startIndex
	 * @param nbMessage
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<Log> findLogFromLogin(String login, int startIndex, int nbMessage) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT log FROM Log log WHERE log.loginAuthor = :loginAuthor ORDER BY log.date DESC");
		
		query.setFirstResult(startIndex);
		query.setMaxResults(nbMessage);
		query.setParameter("loginAuthor", login);
		
		List<Log> listLog = query.list();
		session.close();
		
		return listLog;
	}
	
	/**
	 * Récupère une liste de n Log, dont l'action est "action".
	 * @param action
	 * @param startIndex
	 * @param nbMessage
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<Log> findLogFromAction(String action, int startIndex, int nbMessage) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT log FROM Log log WHERE log.action = :action ORDER BY log.date DESC");
		query.setFirstResult(startIndex);
		query.setMaxResults(nbMessage);
		query.setParameter("action", action);
		
		List<Log> listLog = query.list();
		session.close();
		
		return listLog;
	}
	
	/**
	 * Compte le nombre total de Log enregistré dans la dase de données.
	 * @return
	 */
	public Integer getLogCount(){
		Session session = HibernateUtils.getSessionFactory().openSession();
		Number nbLog = 0;
		try {
			Query query = session.createQuery("SELECT count(log) FROM Log log");
			nbLog = (Number) query.uniqueResult();
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			session.close();
		}
		return nbLog.intValue();
	}

	
	

	/**
	 * Compte le nombre total de Log enregistré au nom de "login" dans la dase de données.
	 * @param login
	 * @return
	 */
	public Integer getLogCountFromLogin(String login){
		Session session = HibernateUtils.getSessionFactory().openSession();
		Number nbLog = 0;
		try {
			Query query = session.createQuery("SELECT count(log) FROM Log log WHERE log.loginAuthor = :loginAuthor");
			query.setParameter("loginAuthor", login);
			nbLog = (Number) query.uniqueResult();
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			session.close();
		}
		return nbLog.intValue();
	}
	
	/**
	 * Compte le nombre total de Log enregistré dont l'action est "action" dans la dase de données.
	 * @param action
	 * @return
	 */
	public Integer getLogCountFromAction(String action){
		Session session = HibernateUtils.getSessionFactory().openSession();
		Number nbLog = 0;
		try {
			Query query = session.createQuery("SELECT count(log) FROM Log log WHERE log.action = :action");
			query.setParameter("action", action);
			nbLog = (Number) query.uniqueResult();
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			session.close();
		}
		return nbLog.intValue();
	}

	// TODO : deleteLogOlderThan(long date){...}
}
