package fr.janua.server.dao.object;

import java.util.List;
import java.util.Set;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import fr.janua.server.PdfManager;
import fr.janua.server.dao.DAO;
import fr.janua.server.entity.Comment;
import fr.janua.server.entity.Message;
import fr.janua.server.entity.Type;
import fr.janua.server.utils.HibernateUtils;

public class MessageDAO extends DAO<Message>{

	public MessageDAO() {
		super();
	}

	@Override
	public boolean create(Message message) {
		initDefaultValueMessage(message);
		message.setLastChange(System.currentTimeMillis());
		Session session = HibernateUtils.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.save(message);
			tx.commit();
		} catch (Exception e) {
			e.printStackTrace();
			if (tx != null && session.isOpen())
				tx.rollback();
			return false;
		}finally{
			session.close();
		}
		return true;
	}

	@Override
	public boolean delete(Message message) {
		// TODO : à coder
		return false;
	}

	@Override
	public boolean update(Message message) {
		message.setLastChange(System.currentTimeMillis());
		Session session = HibernateUtils.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			if(message.getStatus()==0 && message.getClosureDate()==0){
				message.setClosureDate(System.currentTimeMillis());
			}
			session.update(message);
			tx.commit();
		}catch (Exception e) {
			e.printStackTrace();
			if (tx != null && session.isOpen())
				tx.rollback();
			return false;
		}finally{
			session.close();
		}
		return true;
	}

	@Override
	public Message findById(int id) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT message FROM Message message WHERE message.id = :id");
		query.setParameter("id", id);
		Message message = (Message) query.uniqueResult();
		session.close();

		return message;
	}



	/**
	 * Retourne n messages ouverts, à partir de l'index passé en paramètre.
	 * @param startIndex
	 * @param nbMessage
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<Message> findOpenMessages(int startIndex, int nbMessage, List<String> listNameSite, List<String> listNameType){
		if(listNameSite==null || listNameType ==null || listNameSite.size()==0 || listNameType.size()==0){
			return null;
		}else{
			Session session = HibernateUtils.getSessionFactory().openSession();

			Query query = session.createQuery(	"SELECT DISTINCT m FROM Message m JOIN m.types types WHERE " +
					"m.status = :status AND " +
					"m.site.name in (:sites) AND " +
					"types.name in (:types) " +
					"ORDER BY m.lastChange DESC");

			query.setFirstResult(startIndex);
			query.setMaxResults(nbMessage);
			query.setParameter("status", 1);
			query.setParameterList("sites", listNameSite);
			query.setParameterList("types", listNameType);
			List<Message> messages = query.list();
			session.close();

			return messages;
		}
	}

	/**
	 * Retourne n messages fermés, à partir de l'index passé en paramètre. Cette fonction initialise aussi le fullPath du PdfManager.
	 * 
	 * @see PdfManager#getFullPathPDF()
	 * @param startIndex
	 * @param nbMessage
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<Message> findClosedMessages(int startIndex, int nbMessage, List<String> listNameSite, List<String> listNameType) {
		// Initialise le PdfManager pour que le fullPath soit à jour à la première connexion
		PdfManager.getInstance();
		if(listNameSite==null || listNameType ==null || listNameSite.size()==0 || listNameType.size()==0){
			return null;
		}else{
			Session session = HibernateUtils.getSessionFactory().openSession();

			Query query = session.createQuery(	"SELECT DISTINCT m FROM Message m JOIN m.types types WHERE " +
					"m.status = :status AND " +
					"m.site.name in (:sites) AND " +
					"types.name in (:types) " +
					"ORDER BY m.closureDate DESC");

			query.setFirstResult(startIndex);
			query.setMaxResults(nbMessage);
			query.setParameter("status", 0);
			query.setParameterList("sites", listNameSite);
			query.setParameterList("types", listNameType);
			List<Message> messages = query.list();
			session.close();

			return messages;
		}
	}


	// TODO : ajouter des site, des types, un indexStart et nbElement
	@SuppressWarnings("unchecked")
	public List<Message> search(String  wordSearch){
		Session session = HibernateUtils.getSessionFactory().openSession();
	
//		wordSearch = wordSearch.replace(" ", "%");
//		wordSearch = "%" + wordSearch + "%";
	
		//		ArrayList<String> listWord = new ArrayList<String>(Arrays.asList(words));
//		Query query = session.createQuery(	"SELECT DISTINCT m FROM Message m WHERE " +
//				"m.description like (:word) " +
//				"ORDER BY m.id DESC");
//		query.setParameter("word", "%"+wordSearch+"%");
		Query query = session.createQuery(wordSearch);
		List<Message> messages = query.list();
		session.close();
	
		return messages;
	}


	
	/**
	 * Retourne le nombre de Messages ouverts contenu dans la base de donnée.
	 * @return
	 */
	public Integer getOpenMessageCount(List<String> listNameSite, List<String> listNameType) {
		if(listNameSite==null || listNameType ==null || listNameSite.size()==0 || listNameType.size()==0){
			return 0;
		}else{
			Session session = HibernateUtils.getSessionFactory().openSession();
			Integer nbOpenMessage = 0;
			try {
				Query query = session.createQuery(	"SELECT DISTINCT count(DISTINCT m) FROM Message m JOIN m.types types WHERE " +
						"m.status = :status AND " +
						"m.site.name in (:sites) AND " +
						"types.name in (:types)");
				query.setParameter("status", 1);
				query.setParameterList("sites", listNameSite);
				query.setParameterList("types", listNameType);
				nbOpenMessage = ((Number)query.uniqueResult()).intValue();
			} catch (Exception e) {
				e.printStackTrace();
			}finally{
				session.close();
			}

			return nbOpenMessage;
		}
	}

	/**
	 * Retourne le nombre de Messages fermés contenu dans la base de donnée.
	 * @return
	 */
	public Integer getClosedMessageCount(List<String> listNameSite, List<String> listNameType) {
		if(listNameSite==null || listNameType ==null || listNameSite.size()==0 || listNameType.size()==0){
			return 0;
		}else{
			Session session = HibernateUtils.getSessionFactory().openSession();
			Integer nbOpenMessage = 0;
			try {
				Query query = session.createQuery(	"SELECT count(DISTINCT m) FROM Message m JOIN m.types types WHERE " +
						"m.status = :status AND " +
						"m.site.name in (:sites) AND " +
						"types.name in (:types)");
				query.setParameter("status", 0);
				query.setParameterList("sites", listNameSite);
				query.setParameterList("types", listNameType);
				nbOpenMessage = ((Number)query.uniqueResult()).intValue();
			} catch (Exception e) {
				e.printStackTrace();
			}finally{
				session.close();
			}

			return nbOpenMessage;
		}
	}

	/**
	 * Ajout d'un commentaire à un message.
	 * @param idMessage
	 * @param description
	 * @param groupName
	 * @param author
	 * @return
	 */
	public Boolean addComment(int idMessage, String description, String author, String groupName){
		
		Session session = HibernateUtils.getSessionFactory().openSession();

		// le message auquel on va ajouter un commentaire
		Message message = findById(idMessage);
		message.setLastChange(System.currentTimeMillis());
		// si le message est déjà fermé, on ne peut pas le commenté ! 
		if(message.getStatus() == 0){
			return false;
		}

		long time = System.currentTimeMillis();

		Comment c = new Comment();
		c.setAuthor(author);
		c.setGroupName(groupName);
		c.setDescription(description);
		c.setDate(time);

		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.save(c);
			tx.commit();

			Query query2 = session.createQuery("SELECT comment FROM Comment comment WHERE comment.author=:author and comment.groupName=:groupName and comment.description=:description and comment.date=:date");
			query2.setParameter("author", author);
			query2.setParameter("groupName", groupName);
			query2.setParameter("description", description);
			query2.setParameter("date", time);
			Comment commentAdded = (Comment) query2.uniqueResult();

			message.getComments().add(commentAdded);

			update(message);
		} catch (Exception e) {
			e.printStackTrace();
			if (tx != null && session.isOpen())
				tx.rollback();
			return false;
		}finally{
			session.close();
		}

		return true;
	}

	/**
	 * Ajoute un type au message, et ajoute automatiquement un commentaire.
	 * @param idMessage
	 * @param type
	 * @param login
	 * @param groupName
	 * @return
	 */
	public Boolean addType(int idMessage, Type type, String login, String groupName){
		Session session = HibernateUtils.getSessionFactory().openSession();

		// le message auquel on va ajouter un commentaire
		Message message = findById(idMessage);
		message.setLastChange(System.currentTimeMillis());
		
		// si le message est déjà fermé, on ne peut pas ajouter de type !
		if(message.getStatus() == 0){
			return false;
		}

		long time = System.currentTimeMillis();
		
		Comment c = new Comment();
		c.setAuthor(login);
		c.setGroupName(groupName);
		c.setType(type);
		c.setDate(time);

		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			// on enregistre le commentaire...
			session.save(c);
			tx.commit();
			// ... avant de le récupérer
			Query query2 = session.createQuery("SELECT comment FROM Comment comment WHERE comment.author=:author and comment.groupName=:groupName and comment.type=:type and comment.date=:date");
			query2.setParameter("author", login);
			query2.setParameter("groupName", groupName);
			query2.setParameter("type", type);
			query2.setParameter("date", time);
			
			// le commentaire tout juste récupéré
			Comment commentAdded = (Comment) query2.uniqueResult();

			// ajout du nouveau type au message, et association du commentaire au message
			Set<Type> setTypesUpdated = message.getTypes();
			setTypesUpdated.add(type);
			message.setTypes(setTypesUpdated);
			message.getComments().add(commentAdded);
			// puis update du message
			update(message);
		} catch (Exception e) {
			e.printStackTrace();
			if (tx != null && session.isOpen())
				tx.rollback();
			return false;
		}finally{
			session.close();
		}

		return true;
	}


	/**
	 * Initialise la date de création à l'heure actuelle. Si la date de l'événement n'a pas été entré, alors on la définit à l'heure actuelle. 
	 * @param message
	 */
	private void initDefaultValueMessage(Message message){
		message.setCreationDate(System.currentTimeMillis());
		if(message.getEventDate() == 0){
			message.setEventDate(System.currentTimeMillis());
		}
	}


	public Boolean generatePdf(int idMessage){
		// System.out.println("Generate PDF : ");
		// System.out.println("message : " + idMessage);
		Message m = findById(idMessage);

		return PdfManager.getInstance().createPdf(m);
	}
}
