package fr.janua.server.dao.object;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.List;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import fr.janua.server.dao.DAO;
import fr.janua.server.entity.User;
import fr.janua.server.utils.HibernateUtils;

public class UserDAO extends DAO<User>{

	public UserDAO() {
		super();
	}

	@Override
	public boolean create(User user) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		user.setPassword(encryptWithSalt(user.getPassword()));
		user.setLastActivity(0);
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.save(user);
			tx.commit();
		} catch (Exception e) {
			e.printStackTrace();
			if (tx != null && session.isOpen())
				tx.rollback();
			System.out.println("[User] create(" + user.getLogin() + ") : not ok !!");
			return false;
		}finally{
			session.close();
		}
		return true;
	}

	@Override
	public boolean delete(User user) {
		// TODO : à coder
		return false;
	}

	@Override
	public boolean update(User user) {
		// si le mot de passe n'a pas changé, on récupère l'ancien mot de passe
		Session session = HibernateUtils.getSessionFactory().openSession();
		if(user.getPassword() == null || user.getPassword().equals("")){
			String oldPassword = getPassword(session, user.getLogin());
			if(oldPassword==null){
				session.close();
				return false;
			}else{
				user.setPassword(oldPassword);
			}
		}else{
			user.setPassword(encryptWithSalt(user.getPassword()));
		}
		
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.update(user);
			tx.commit();
		}catch (Exception e) {
			e.printStackTrace();
			if (tx != null && session.isOpen())
				tx.rollback();
			return false;
		}finally{
			session.close();
		}
		return true;
	}

	@Override
	public User findById(int id) {
		System.err.println("User n'a pas d'id. Utiliser la fonction finByName !");
		return null;
	}
	
	public User findByLogin(String login) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT user FROM User user WHERE user.login = :login");
		query.setParameter("login", login);
		User user = (User) query.uniqueResult();
		session.close();
		
		return user;
	}
	
	/**
	 * Retourne le mot de passe d'un utilisateur grace à son login.
	 * A chaque fois que le client récupère des utilisateurs, on masque les mots de passe.
	 * Lorsque le client fait un "update" de l'utilisateur, il y aura deux possibilités :
	 * <ul>
	 * 	<li>il a changé le mot de passe : on enregistre directement dans la BDD le nouvel User</li>
	 * 	<li>il n'a pas changé le mot de passe : on va récupérer l'ancien dans la BDD, puis on enregistre ce nouvel User</li>
	 * </ul> 
	 * @see UserDAO#update(User)
	 * @param s
	 * @param login
	 * @return
	 */
	private String getPassword(Session s, String login) {
		if(s.isOpen()){
			Query query = s.createQuery("SELECT user.password FROM User user WHERE user.login = :login");
			query.setParameter("login", login);
			String password = (String) query.uniqueResult();
			return password;
		}else{
			return null;
		}
	}

	/**
	 * Récupère tous les Users de la base de données.
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<User> findAllUser() {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT user FROM User user");
		List<User> list = query.list();
		// on efface les mots de passe
		for(User user : list)
			user.setPassword("");
		session.close();
		
		return list;
	}
	
	/**
	 * Récupère tous les login des Users de la base de données.
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<String> findAllLoginUser() {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT user.login FROM User user");
		List<String> list = query.list();
		session.close();
		
		return list;
	}

	
	/**
	 * 
	 * @param indexStart
	 * @param nbElement
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<User> findUser(int indexStart, int nbElement) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		List<User> listUser = null;
		try {
			Query query = session.createQuery("SELECT user FROM User user ORDER BY user.login ASC");
			query.setFirstResult(indexStart);
			query.setMaxResults(nbElement);
			listUser = query.list();
			// on efface les mots de passe
			for(User user : listUser)
				user.setPassword("");
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			session.close();
		}
		return listUser;
	}
	
	/**
	 * Retourne le nombre de User contenu dans la base de donnée.
	 * @return
	 */
	public int getUserCount() {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Number nbUser = 0;
		try {
			Query query = session.createQuery("SELECT count(user) FROM User user");
			nbUser = (Number) query.uniqueResult();
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			session.close();
		}
		return nbUser.intValue();
	}
	
	/**
	 * Répond <code>true</code> si le login est disponible.
	 * @param login
	 * @return
	 */
	public boolean checkAvailability(String login) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT user.login FROM User user WHERE user.login = :login");
		query.setParameter("login", login);
		String loginResult = (String) query.uniqueResult();
		session.close();
		boolean available = (loginResult==null) ? true : false;
		return available;
	}
	
	/**
	 * Vérifie que le mot de passe et le login concordent avec un compte existant.
	 * Cette méthode encrypte le mot de passe avant de la comparer avec celle de la base de données.
	 * 
	 * @see UserDAO#checkWithEncryptedPaswword(String, String)
	 * @param login
	 * @param password
	 * @return
	 */
	public User check(String login, String password){
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT user FROM User user WHERE user.login = :login");
		query.setParameter("login", login);
		User user = (User) query.uniqueResult();
		session.close();
		if(user == null){
			// si login innexistant
			return null;
		}else if(user.getPassword().equals(encryptWithSalt(password))){
			// si bon mot de passe / login
			return user;
		}else{
			// si mauvais mot de passe / login
			return null;
		}
	}
	
	/**
	 * Vérifie que le mot de passe et le login concordent avec un compte existant. On check aussi que l'utilisateur est bien actif.
	 * Cette méthode n'encrypte pas le mot de passe avant de la comparer avec celle de la base de données.
	 * 
	 * @param login
	 * @param password
	 * @return
	 */
	public User checkWithEncryptedPaswword(String login, String password){
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT user FROM User user WHERE user.login = :login");
		query.setParameter("login", login);
		User user = (User) query.uniqueResult();
		session.close();
		if(user.getPassword().equals(password) && user.isEnabled()){
			return user;
		}else{
			return null;
		}
	}
	
	/**
	 * Retourne <code>true</code> si un User ayant les droits ADMIN exist.
	 * @return
	 */
	public boolean isAdminExist(){
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery(	"SELECT u FROM User u JOIN u.group g WHERE g.crudGroup = :crudGroup");
		query.setParameter("crudGroup", 7);
		User user = (User) query.uniqueResult();
		session.close();
		if(user == null)
			return false;
		else
			return true;
	}

	/**
	 * Change la date de dernière activité.
	 * @param login
	 */
	public boolean updateLastActivity(String login){
		Session session = HibernateUtils.getSessionFactory().openSession();
		
		Query query = session.createQuery("SELECT user FROM User user WHERE user.login = :login");
		query.setParameter("login", login);
		User user = (User) query.uniqueResult();
		user.setLastActivity(System.currentTimeMillis());
		
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.update(user);
			tx.commit();
		}catch (Exception e) {
			e.printStackTrace();
			if (tx != null && session.isOpen())
				tx.rollback();
			return false;
		}finally{
			session.close();
		}
		return true;
	}
	
	/**
	 * Méthode sécurisé pour encrypté de manière forte le mot de passe avec un grain de sel (ou de sable, au choix...).
	 * MD5( MD5(salt+pswd) + MD5(salt+salt) )
	 * @param password
	 * @return
	 */
	private String encryptWithSalt(String password){
		String salt = "hello";
		String passwordEncrypted = encryptMD5(encryptMD5(salt+password)+encryptMD5(salt+salt));
		return passwordEncrypted;
	}


	/**
	 * Encrypte une chaîne de caractère en MD5.
	 * @param messageToEncode
	 * @return
	 */
	private String encryptMD5(String messageToEncode){
		byte[] bytesOfMessage = null;
		try {
			bytesOfMessage = messageToEncode.getBytes("UTF-8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}

		MessageDigest md = null;
		try {
			md = MessageDigest.getInstance("MD5");
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		}
		byte[] bytesOfHash = md.digest(bytesOfMessage);

		StringBuilder messageHashed = new StringBuilder();
		for (int i = 0; i < bytesOfHash.length; i++){
			String hex = Integer.toHexString(bytesOfHash[i]);
			if (hex.length() == 1){
				messageHashed.append('0');
				messageHashed.append(hex.charAt(hex.length() - 1));
			}else{
				messageHashed.append(hex.substring(hex.length() - 2));
			}
		}

		return messageHashed.toString();
	}



}
